@@ -1,5 +1,5 @@
 /*******************************************************************************
- Copyright (c) 2012-2020, Intel Corporation
+ Copyright (c) 2012-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -71,10 +71,19 @@ IMB_JOB *submit_job_zuc_eea3_avx2(MB_MGR_ZUC_OOO *state,
                                         IMB_JOB *job);
 IMB_JOB *flush_job_zuc_eea3_avx2(MB_MGR_ZUC_OOO *state);
 
+IMB_JOB *flush_job_zuc256_eea3_avx2(MB_MGR_ZUC_OOO *state);
+
+IMB_JOB *submit_job_zuc256_eea3_avx2(MB_MGR_ZUC_OOO *state,
+                                     IMB_JOB *job);
+
 IMB_JOB *submit_job_zuc_eia3_avx2(MB_MGR_ZUC_OOO *state,
                                         IMB_JOB *job);
 IMB_JOB *flush_job_zuc_eia3_avx2(MB_MGR_ZUC_OOO *state);
 
+IMB_JOB *submit_job_zuc256_eia3_avx2(MB_MGR_ZUC_OOO *state,
+                                        IMB_JOB *job);
+IMB_JOB *flush_job_zuc256_eia3_avx2(MB_MGR_ZUC_OOO *state);
+
 void aes_cmac_256_subkey_gen_avx2(const void *key_exp,
                                   void *key1, void *key2);
 uint32_t hec_32_avx(const uint8_t *in);
@@ -84,7 +93,12 @@ IMB_JOB *submit_job_aes128_cbcs_1_9_enc_avx(MB_MGR_AES_OOO *state,
                                             IMB_JOB *job);
 IMB_JOB *flush_job_aes128_cbcs_1_9_enc_avx(MB_MGR_AES_OOO *state);
 
-IMB_JOB *submit_job_chacha20_enc_dec_avx(IMB_JOB *job);
+IMB_JOB *submit_job_chacha20_enc_dec_avx2(IMB_JOB *job);
+
+void *poly1305_mac_scalar(IMB_JOB *job);
+
+IMB_JOB *snow_v_sse(IMB_JOB *job);
+IMB_JOB *snow_v_aead_init_sse(IMB_JOB *job);
 
 #define SAVE_XMMS               save_xmms_avx
 #define RESTORE_XMMS            restore_xmms_avx
@@ -115,6 +129,10 @@ IMB_JOB *submit_job_chacha20_enc_dec_avx(IMB_JOB *job);
 #define FLUSH_JOB_ZUC_EEA3    flush_job_zuc_eea3_avx2
 #define SUBMIT_JOB_ZUC_EIA3   submit_job_zuc_eia3_avx2
 #define FLUSH_JOB_ZUC_EIA3    flush_job_zuc_eia3_avx2
+#define SUBMIT_JOB_ZUC256_EEA3   submit_job_zuc256_eea3_avx2
+#define FLUSH_JOB_ZUC256_EEA3    flush_job_zuc256_eea3_avx2
+#define SUBMIT_JOB_ZUC256_EIA3   submit_job_zuc256_eia3_avx2
+#define FLUSH_JOB_ZUC256_EIA3    flush_job_zuc256_eia3_avx2
 
 #define AES_CBC_DEC_128       aes_cbc_dec_128_avx
 #define AES_CBC_DEC_192       aes_cbc_dec_192_avx
@@ -154,9 +172,7 @@ IMB_JOB *submit_job_chacha20_enc_dec_avx(IMB_JOB *job);
 #define AES_GCM_ENC_IV_256   aes_gcm_enc_var_iv_256_avx_gen4
 
 #define SUBMIT_JOB_AES_GCM_DEC submit_job_aes_gcm_dec_avx2
-#define FLUSH_JOB_AES_GCM_DEC  flush_job_aes_gcm_dec_avx2
 #define SUBMIT_JOB_AES_GCM_ENC submit_job_aes_gcm_enc_avx2
-#define FLUSH_JOB_AES_GCM_ENC  flush_job_aes_gcm_enc_avx2
 
 #define SUBMIT_JOB_AES_XCBC   submit_job_aes_xcbc_avx
 #define FLUSH_JOB_AES_XCBC    flush_job_aes_xcbc_avx
@@ -170,8 +186,13 @@ IMB_JOB *submit_job_chacha20_enc_dec_avx(IMB_JOB *job);
 #define FLUSH_JOB_AES_ENC  FLUSH_JOB_AES_ENC_AVX2
 #define SUBMIT_JOB_AES_DEC SUBMIT_JOB_AES_DEC_AVX2
 
-#define SUBMIT_JOB_CHACHA20_ENC_DEC submit_job_chacha20_enc_dec_avx
-#define SUBMIT_JOB_CHACHA20_POLY1305 aead_chacha20_poly1305_avx
+#define SUBMIT_JOB_CHACHA20_ENC_DEC submit_job_chacha20_enc_dec_avx2
+#define SUBMIT_JOB_CHACHA20_POLY1305 aead_chacha20_poly1305_avx2
+#define SUBMIT_JOB_CHACHA20_POLY1305_SGL aead_chacha20_poly1305_sgl_avx2
+#define POLY1305_MAC poly1305_mac_scalar
+
+#define SUBMIT_JOB_SNOW_V snow_v_sse
+#define SUBMIT_JOB_SNOW_V_AEAD snow_v_aead_init_sse
 
 IMB_JOB *submit_job_hmac_avx2(MB_MGR_HMAC_SHA_1_OOO *state,
                                    IMB_JOB *job);
@@ -339,18 +360,10 @@ submit_job_aes_gcm_dec_avx2(IMB_MGR *state, IMB_JOB *job)
                                    job->auth_tag_output_len_in_bytes);
         }
 
-        job->status = STS_COMPLETED;
+        job->status = IMB_STATUS_COMPLETED;
         return job;
 }
 
-static IMB_JOB *
-flush_job_aes_gcm_dec_avx2(IMB_MGR *state, IMB_JOB *job)
-{
-        (void) state;
-        (void) job;
-        return NULL;
-}
-
 static IMB_JOB *
 submit_job_aes_gcm_enc_avx2(IMB_MGR *state, IMB_JOB *job)
 {
@@ -392,18 +405,10 @@ submit_job_aes_gcm_enc_avx2(IMB_MGR *state, IMB_JOB *job)
                                    job->auth_tag_output_len_in_bytes);
         }
 
-        job->status = STS_COMPLETED;
+        job->status = IMB_STATUS_COMPLETED;
         return job;
 }
 
-static IMB_JOB *
-flush_job_aes_gcm_enc_avx2(IMB_MGR *state, IMB_JOB *job)
-{
-        (void) state;
-        (void) job;
-        return NULL;
-}
-
 /* ====================================================================== */
 
 void
@@ -439,7 +444,9 @@ init_mb_mgr_avx2(IMB_MGR *state)
         MB_MGR_CCM_OOO *aes_ccm_ooo = state->aes_ccm_ooo;
         MB_MGR_CMAC_OOO *aes_cmac_ooo = state->aes_cmac_ooo;
         MB_MGR_ZUC_OOO *zuc_eea3_ooo = state->zuc_eea3_ooo;
+        MB_MGR_ZUC_OOO *zuc256_eea3_ooo = state->zuc256_eea3_ooo;
         MB_MGR_ZUC_OOO *zuc_eia3_ooo = state->zuc_eia3_ooo;
+        MB_MGR_ZUC_OOO *zuc256_eia3_ooo = state->zuc256_eia3_ooo;
         MB_MGR_AES_OOO *aes128_cbcs_ooo = state->aes128_cbcs_ooo;
 
         /* reset error status */
@@ -543,6 +550,27 @@ init_mb_mgr_avx2(IMB_MGR *state)
         zuc_eia3_ooo->init_not_done = 0;
         zuc_eia3_ooo->unused_lane_bitmask = 0xff;
 
+        memset(zuc256_eea3_ooo->lens, 0,
+               sizeof(zuc256_eea3_ooo->lens));
+        memset(zuc256_eea3_ooo->job_in_lane, 0,
+               sizeof(zuc256_eea3_ooo->job_in_lane));
+        zuc256_eea3_ooo->unused_lanes = 0xF76543210;
+        zuc256_eea3_ooo->num_lanes_inuse = 0;
+        memset(&zuc256_eea3_ooo->state, 0,
+               sizeof(zuc256_eea3_ooo->state));
+        zuc256_eea3_ooo->init_not_done = 0;
+        zuc256_eea3_ooo->unused_lane_bitmask = 0xff;
+
+        memset(zuc256_eia3_ooo->lens, 0xFF,
+               sizeof(zuc256_eia3_ooo->lens));
+        memset(zuc256_eia3_ooo->job_in_lane, 0,
+               sizeof(zuc256_eia3_ooo->job_in_lane));
+        zuc256_eia3_ooo->unused_lanes = 0xF76543210;
+        zuc256_eia3_ooo->num_lanes_inuse = 0;
+        memset(&zuc256_eia3_ooo->state, 0,
+               sizeof(zuc256_eia3_ooo->state));
+        zuc256_eia3_ooo->init_not_done = 0;
+        zuc256_eia3_ooo->unused_lane_bitmask = 0xff;
 
         /* Init HMAC/SHA1 out-of-order fields */
         hmac_sha_1_ooo->lens[0] = 0;
@@ -637,16 +665,16 @@ init_mb_mgr_avx2(IMB_MGR *state)
                 MB_MGR_HMAC_SHA_512_OOO *ctx = hmac_sha_384_ooo;
 
                 ctx->ldata[j].job_in_lane = NULL;
-                ctx->ldata[j].extra_block[SHA_384_BLOCK_SIZE] = 0x80;
-                memset(ctx->ldata[j].extra_block + (SHA_384_BLOCK_SIZE + 1),
-                       0x00, SHA_384_BLOCK_SIZE + 7);
+                ctx->ldata[j].extra_block[IMB_SHA_384_BLOCK_SIZE] = 0x80;
+                memset(ctx->ldata[j].extra_block + (IMB_SHA_384_BLOCK_SIZE + 1),
+                       0x00, IMB_SHA_384_BLOCK_SIZE + 7);
                 p = ctx->ldata[j].outer_block;
                 /* special end point because this length is constant */
-                memset(p + SHA384_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
-                       SHA_384_BLOCK_SIZE -
-                       SHA384_DIGEST_SIZE_IN_BYTES  - 1 - 2);
+                memset(p + IMB_SHA384_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
+                       IMB_SHA_384_BLOCK_SIZE -
+                       IMB_SHA384_DIGEST_SIZE_IN_BYTES  - 1 - 2);
                 /* mark the end */
-                p[SHA384_DIGEST_SIZE_IN_BYTES] = 0x80;
+                p[IMB_SHA384_DIGEST_SIZE_IN_BYTES] = 0x80;
                 /* hmac outer block length always of fixed size,
                  * it is OKey length, a whole message block length, 1024 bits,
                  * with padding plus the length of the inner digest,
@@ -654,8 +682,8 @@ init_mb_mgr_avx2(IMB_MGR *state)
                  * The input message block needs to be converted to big endian
                  * within the sha implementation before use.
                  */
-                p[SHA_384_BLOCK_SIZE - 2] = 0x05;
-                p[SHA_384_BLOCK_SIZE - 1] = 0x80;
+                p[IMB_SHA_384_BLOCK_SIZE - 2] = 0x05;
+                p[IMB_SHA_384_BLOCK_SIZE - 1] = 0x80;
         }
 
         /* Init HMAC/SHA512 out-of-order fields */
@@ -672,16 +700,16 @@ init_mb_mgr_avx2(IMB_MGR *state)
                 MB_MGR_HMAC_SHA_512_OOO *ctx = hmac_sha_512_ooo;
 
                 ctx->ldata[j].job_in_lane = NULL;
-                ctx->ldata[j].extra_block[SHA_512_BLOCK_SIZE] = 0x80;
-                memset(ctx->ldata[j].extra_block + (SHA_512_BLOCK_SIZE + 1),
-                       0x00, SHA_512_BLOCK_SIZE + 7);
+                ctx->ldata[j].extra_block[IMB_SHA_512_BLOCK_SIZE] = 0x80;
+                memset(ctx->ldata[j].extra_block + (IMB_SHA_512_BLOCK_SIZE + 1),
+                       0x00, IMB_SHA_512_BLOCK_SIZE + 7);
                 p = ctx->ldata[j].outer_block;
                 /* special end point because this length is constant */
-                memset(p + SHA512_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
-                       SHA_512_BLOCK_SIZE -
-                       SHA512_DIGEST_SIZE_IN_BYTES  - 1 - 2);
+                memset(p + IMB_SHA512_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
+                       IMB_SHA_512_BLOCK_SIZE -
+                       IMB_SHA512_DIGEST_SIZE_IN_BYTES  - 1 - 2);
                 /* mark the end */
-                p[SHA512_DIGEST_SIZE_IN_BYTES] = 0x80;
+                p[IMB_SHA512_DIGEST_SIZE_IN_BYTES] = 0x80;
                 /* hmac outer block length always of fixed size,
                  * it is OKey length, a whole message block length, 1024 bits,
                  * with padding plus the length of the inner digest,
@@ -689,8 +717,8 @@ init_mb_mgr_avx2(IMB_MGR *state)
                  * The input message block needs to be converted to big endian
                  * within the sha implementation before use.
                  */
-                p[SHA_512_BLOCK_SIZE - 2] = 0x06;
-                p[SHA_512_BLOCK_SIZE - 1] = 0x00;
+                p[IMB_SHA_512_BLOCK_SIZE - 2] = 0x06;
+                p[IMB_SHA_512_BLOCK_SIZE - 1] = 0x00;
         }
 
         /* Init HMAC/MD5 out-of-order fields */
@@ -888,6 +916,11 @@ init_mb_mgr_avx2(IMB_MGR *state)
         state->crc6_iuup_header    = crc6_iuup_header_avx;
         state->crc32_wimax_ofdma_data = crc32_wimax_ofdma_data_avx;
         state->crc8_wimax_ofdma_hcs = crc8_wimax_ofdma_hcs_avx;
+
+        state->chacha20_poly1305_init = init_chacha20_poly1305_avx;
+        state->chacha20_poly1305_enc_update = update_enc_chacha20_poly1305_avx2;
+        state->chacha20_poly1305_dec_update = update_dec_chacha20_poly1305_avx2;
+        state->chacha20_poly1305_finalize = finalize_chacha20_poly1305_avx;
 }
 
 #include "mb_mgr_code.h"
