@@ -1,5 +1,5 @@
 /*******************************************************************************
- Copyright (c) 2012-2020, Intel Corporation
+ Copyright (c) 2012-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -127,10 +127,26 @@ IMB_JOB *submit_job_zuc_eia3_gfni_avx512(MB_MGR_ZUC_OOO *state,
                                               IMB_JOB *job);
 IMB_JOB *flush_job_zuc_eia3_gfni_avx512(MB_MGR_ZUC_OOO *state);
 
+IMB_JOB *submit_job_zuc256_eea3_no_gfni_avx512(MB_MGR_ZUC_OOO *state,
+                                                 IMB_JOB *job);
+IMB_JOB *flush_job_zuc256_eea3_no_gfni_avx512(MB_MGR_ZUC_OOO *state);
+
+IMB_JOB *submit_job_zuc256_eea3_gfni_avx512(MB_MGR_ZUC_OOO *state,
+                                              IMB_JOB *job);
+IMB_JOB *flush_job_zuc256_eea3_gfni_avx512(MB_MGR_ZUC_OOO *state);
+
 IMB_JOB *aes_cntr_ccm_128_vaes_avx512(IMB_JOB *job);
 
 IMB_JOB *aes_cntr_ccm_256_vaes_avx512(IMB_JOB *job);
 
+IMB_JOB *submit_job_zuc256_eia3_no_gfni_avx512(MB_MGR_ZUC_OOO *state,
+                                        IMB_JOB *job);
+IMB_JOB *flush_job_zuc256_eia3_no_gfni_avx512(MB_MGR_ZUC_OOO *state);
+
+IMB_JOB *submit_job_zuc256_eia3_gfni_avx512(MB_MGR_ZUC_OOO *state,
+                                        IMB_JOB *job);
+IMB_JOB *flush_job_zuc256_eia3_gfni_avx512(MB_MGR_ZUC_OOO *state);
+
 void aes_cmac_256_subkey_gen_avx512(const void *key_exp,
                                     void *key1, void *key2);
 uint32_t hec_32_avx(const uint8_t *in);
@@ -144,7 +160,8 @@ IMB_JOB *submit_job_aes128_cbcs_1_9_enc_vaes_avx512(MB_MGR_AES_OOO *state,
                                                IMB_JOB *job);
 IMB_JOB *flush_job_aes128_cbcs_1_9_enc_vaes_avx512(MB_MGR_AES_OOO *state);
 
-
+IMB_JOB *snow_v_sse(IMB_JOB *job);
+IMB_JOB *snow_v_aead_init_sse(IMB_JOB *job);
 
 #define SAVE_XMMS               save_xmms_avx
 #define RESTORE_XMMS            restore_xmms_avx
@@ -175,6 +192,10 @@ IMB_JOB *flush_job_aes128_cbcs_1_9_enc_vaes_avx512(MB_MGR_AES_OOO *state);
 #define FLUSH_JOB_ZUC_EEA3    flush_job_zuc_eea3_avx512
 #define SUBMIT_JOB_ZUC_EIA3   submit_job_zuc_eia3_avx512
 #define FLUSH_JOB_ZUC_EIA3    flush_job_zuc_eia3_avx512
+#define SUBMIT_JOB_ZUC256_EEA3   submit_job_zuc256_eea3_avx512
+#define FLUSH_JOB_ZUC256_EEA3    flush_job_zuc256_eea3_avx512
+#define SUBMIT_JOB_ZUC256_EIA3   submit_job_zuc256_eia3_avx512
+#define FLUSH_JOB_ZUC256_EIA3    flush_job_zuc256_eia3_avx512
 
 #define AES_CBC_DEC_128       aes_cbc_dec_128_avx512
 #define AES_CBC_DEC_192       aes_cbc_dec_192_avx512
@@ -226,6 +247,11 @@ IMB_JOB *flush_job_aes128_cbcs_1_9_enc_vaes_avx512(MB_MGR_AES_OOO *state);
 
 #define SUBMIT_JOB_CHACHA20_ENC_DEC submit_job_chacha20_enc_dec_avx512
 #define SUBMIT_JOB_CHACHA20_POLY1305 aead_chacha20_poly1305_avx512
+#define SUBMIT_JOB_CHACHA20_POLY1305_SGL aead_chacha20_poly1305_sgl_avx512
+#define POLY1305_MAC poly1305_mac_avx512
+
+#define SUBMIT_JOB_SNOW_V snow_v_sse
+#define SUBMIT_JOB_SNOW_V_AEAD snow_v_aead_init_sse
 
 IMB_JOB *submit_job_hmac_avx512(MB_MGR_HMAC_SHA_1_OOO *state,
                                      IMB_JOB *job);
@@ -294,6 +320,12 @@ IMB_JOB *flush_job_aes256_ccm_auth_vaes_avx512(MB_MGR_CCM_OOO *state);
 
 IMB_JOB *submit_job_chacha20_enc_dec_avx512(IMB_JOB *job);
 
+void poly1305_mac_fma_avx512(IMB_JOB *job);
+void poly1305_mac_plain_avx512(IMB_JOB *job);
+
+static void (*poly1305_mac_avx512)
+        (IMB_JOB *) = poly1305_mac_plain_avx512;
+
 __forceinline
 IMB_JOB *
 SUBMIT_JOB_DOCSIS_SEC_CRC_ENC(MB_MGR_DOCSIS_AES_OOO *state, IMB_JOB *job,
@@ -350,9 +382,7 @@ SUBMIT_JOB_DOCSIS_SEC_CRC_DEC(MB_MGR_DOCSIS_AES_OOO *state, IMB_JOB *job,
 #define AES_GCM_ENC_IV_256_VAES aes_gcm_enc_var_iv_256_vaes_avx512
 
 #define SUBMIT_JOB_AES_GCM_DEC submit_job_aes_gcm_dec_avx512
-#define FLUSH_JOB_AES_GCM_DEC  flush_job_aes_gcm_avx512
 #define SUBMIT_JOB_AES_GCM_ENC submit_job_aes_gcm_enc_avx512
-#define FLUSH_JOB_AES_GCM_ENC  flush_job_aes_gcm_avx512
 
 /* ====================================================================== */
 
@@ -477,7 +507,7 @@ plain_submit_gcm_dec_avx512(IMB_MGR *state, IMB_JOB *job)
                                    job->auth_tag_output_len_in_bytes);
         }
 
-        job->status = STS_COMPLETED;
+        job->status = IMB_STATUS_COMPLETED;
         return job;
 }
 
@@ -522,7 +552,7 @@ plain_submit_gcm_enc_avx512(IMB_MGR *state, IMB_JOB *job)
                                    job->auth_tag_output_len_in_bytes);
         }
 
-        job->status = STS_COMPLETED;
+        job->status = IMB_STATUS_COMPLETED;
         return job;
 }
 
@@ -566,7 +596,7 @@ vaes_submit_gcm_dec_avx512(IMB_MGR *state, IMB_JOB *job)
                                         job->auth_tag_output,
                                         job->auth_tag_output_len_in_bytes);
 
-        job->status = STS_COMPLETED;
+        job->status = IMB_STATUS_COMPLETED;
         return job;
 }
 
@@ -610,18 +640,10 @@ vaes_submit_gcm_enc_avx512(IMB_MGR *state, IMB_JOB *job)
                                         job->auth_tag_output,
                                         job->auth_tag_output_len_in_bytes);
 
-        job->status = STS_COMPLETED;
+        job->status = IMB_STATUS_COMPLETED;
         return job;
 }
 
-static IMB_JOB *
-flush_job_aes_gcm_avx512(IMB_MGR *state, IMB_JOB *job)
-{
-        (void) state;
-        (void) job;
-        return NULL;
-}
-
 static IMB_JOB *(*submit_job_aes_gcm_enc_avx512)
         (IMB_MGR *state, IMB_JOB *job) = plain_submit_gcm_enc_avx512;
 
@@ -643,7 +665,7 @@ vaes_submit_cntr_avx512(IMB_JOB *job)
         else /* assume 32 bytes */
                 aes_cntr_256_submit_vaes_avx512(job);
 
-        job->status |= STS_COMPLETED_AES;
+        job->status |= IMB_STATUS_COMPLETED_CIPHER;
         return job;
 }
 
@@ -657,7 +679,7 @@ vaes_submit_cntr_bit_avx512(IMB_JOB *job)
         else /* assume 32 bytes */
                 aes_cntr_bit_256_submit_vaes_avx512(job);
 
-        job->status |= STS_COMPLETED_AES;
+        job->status |= IMB_STATUS_COMPLETED_CIPHER;
         return job;
 }
 
@@ -751,6 +773,15 @@ static IMB_JOB *
 (*flush_job_zuc_eea3_avx512)
         (MB_MGR_ZUC_OOO *state) = flush_job_zuc_eea3_no_gfni_avx512;
 
+static IMB_JOB *
+(*submit_job_zuc256_eea3_avx512)
+        (MB_MGR_ZUC_OOO *state, IMB_JOB *job) =
+                        submit_job_zuc256_eea3_no_gfni_avx512;
+
+static IMB_JOB *
+(*flush_job_zuc256_eea3_avx512)
+        (MB_MGR_ZUC_OOO *state) = flush_job_zuc256_eea3_no_gfni_avx512;
+
 static IMB_JOB *
 (*submit_job_zuc_eia3_avx512)
         (MB_MGR_ZUC_OOO *state, IMB_JOB *job) =
@@ -760,6 +791,15 @@ static IMB_JOB *
 (*flush_job_zuc_eia3_avx512)
         (MB_MGR_ZUC_OOO *state) = flush_job_zuc_eia3_no_gfni_avx512;
 
+static IMB_JOB *
+(*submit_job_zuc256_eia3_avx512)
+        (MB_MGR_ZUC_OOO *state, IMB_JOB *job) =
+                        submit_job_zuc256_eia3_no_gfni_avx512;
+
+static IMB_JOB *
+(*flush_job_zuc256_eia3_avx512)
+        (MB_MGR_ZUC_OOO *state) = flush_job_zuc256_eia3_no_gfni_avx512;
+
 static IMB_JOB *
 (*submit_job_aes_xcbc_avx512)
         (MB_MGR_AES_XCBC_OOO *state,
@@ -782,7 +822,8 @@ static IMB_JOB *
 static void
 (*aes_cbcs_1_9_dec_128_avx512) (const void *in, const uint8_t *IV,
                                 const void *keys, void *out,
-                                uint64_t len_bytes) = aes_cbcs_1_9_dec_128_avx;
+                                uint64_t len_bytes,
+                                void *next_iv) = aes_cbcs_1_9_dec_128_avx;
 
 /* ====================================================================== */
 
@@ -809,7 +850,7 @@ submit_aes_docsis128_dec_crc32_avx512(MB_MGR_DOCSIS_AES_OOO *state,
         if (job->msg_len_to_hash_in_bytes == 0) {
                 if (job->msg_len_to_cipher_in_bytes == 0) {
                         /* NO cipher, NO CRC32 */
-                        job->status |= STS_COMPLETED_AES;
+                        job->status |= IMB_STATUS_COMPLETED_CIPHER;
                         return job;
                 }
 
@@ -832,7 +873,7 @@ submit_aes_docsis256_dec_crc32_avx512(MB_MGR_DOCSIS_AES_OOO *state,
         if (job->msg_len_to_hash_in_bytes == 0) {
                 if (job->msg_len_to_cipher_in_bytes == 0) {
                         /* NO cipher, NO CRC32 */
-                        job->status |= STS_COMPLETED_AES;
+                        job->status |= IMB_STATUS_COMPLETED_CIPHER;
                         return job;
                 }
 
@@ -879,7 +920,7 @@ submit_job_docsis128_sec_crc_dec_vaes_avx512(MB_MGR_DOCSIS_AES_OOO *state,
         if (job->msg_len_to_hash_in_bytes == 0) {
                 if (job->msg_len_to_cipher_in_bytes == 0) {
                         /* NO cipher, NO CRC32 */
-                        job->status |= STS_COMPLETED_AES;
+                        job->status |= IMB_STATUS_COMPLETED_CIPHER;
                         return job;
                 }
 
@@ -902,7 +943,7 @@ submit_job_docsis256_sec_crc_dec_vaes_avx512(MB_MGR_DOCSIS_AES_OOO *state,
         if (job->msg_len_to_hash_in_bytes == 0) {
                 if (job->msg_len_to_cipher_in_bytes == 0) {
                         /* NO cipher, NO CRC32 */
-                        job->status |= STS_COMPLETED_AES;
+                        job->status |= IMB_STATUS_COMPLETED_CIPHER;
                         return job;
                 }
 
@@ -996,6 +1037,8 @@ init_mb_mgr_avx512(IMB_MGR *state)
         MB_MGR_CMAC_OOO *aes_cmac_ooo = state->aes_cmac_ooo;
         MB_MGR_ZUC_OOO *zuc_eea3_ooo = state->zuc_eea3_ooo;
         MB_MGR_ZUC_OOO *zuc_eia3_ooo = state->zuc_eia3_ooo;
+        MB_MGR_ZUC_OOO *zuc256_eea3_ooo = state->zuc256_eea3_ooo;
+        MB_MGR_ZUC_OOO *zuc256_eia3_ooo = state->zuc256_eia3_ooo;
         MB_MGR_AES_OOO *aes128_cbcs_ooo = state->aes128_cbcs_ooo;
 
         /* reset error status */
@@ -1074,6 +1117,14 @@ init_mb_mgr_avx512(IMB_MGR *state)
                 flush_job_zuc_eea3_avx512 = flush_job_zuc_eea3_gfni_avx512;
                 submit_job_zuc_eia3_avx512 = submit_job_zuc_eia3_gfni_avx512;
                 flush_job_zuc_eia3_avx512 = flush_job_zuc_eia3_gfni_avx512;
+                submit_job_zuc256_eea3_avx512 =
+                                submit_job_zuc256_eea3_gfni_avx512;
+                flush_job_zuc256_eea3_avx512 =
+                                flush_job_zuc256_eea3_gfni_avx512;
+                submit_job_zuc256_eia3_avx512 =
+                                submit_job_zuc256_eia3_gfni_avx512;
+                flush_job_zuc256_eia3_avx512 =
+                                flush_job_zuc256_eia3_gfni_avx512;
         }
 
         /* Init AES out-of-order fields */
@@ -1286,6 +1337,30 @@ init_mb_mgr_avx512(IMB_MGR *state)
         zuc_eia3_ooo->unused_lane_bitmask = 0xffff;
         memset(zuc_eia3_ooo->args.digest, 0, sizeof(zuc_eia3_ooo->args.digest));
 
+        memset(zuc256_eea3_ooo->lens, 0,
+               sizeof(zuc256_eea3_ooo->lens));
+        memset(zuc256_eea3_ooo->job_in_lane, 0,
+               sizeof(zuc256_eea3_ooo->job_in_lane));
+        zuc256_eea3_ooo->unused_lanes = 0xFEDCBA9876543210;
+        zuc256_eea3_ooo->num_lanes_inuse = 0;
+        memset(&zuc256_eea3_ooo->state, 0,
+               sizeof(zuc256_eea3_ooo->state));
+        zuc256_eea3_ooo->init_not_done = 0;
+        zuc256_eea3_ooo->unused_lane_bitmask = 0xffff;
+
+        memset(zuc256_eia3_ooo->lens, 0xFF,
+               sizeof(zuc256_eia3_ooo->lens));
+        memset(zuc256_eia3_ooo->job_in_lane, 0,
+               sizeof(zuc256_eia3_ooo->job_in_lane));
+        zuc256_eia3_ooo->unused_lanes = 0xFEDCBA9876543210;
+        zuc256_eia3_ooo->num_lanes_inuse = 0;
+        memset(&zuc256_eia3_ooo->state, 0,
+               sizeof(zuc256_eia3_ooo->state));
+        zuc256_eia3_ooo->init_not_done = 0;
+        zuc256_eia3_ooo->unused_lane_bitmask = 0xffff;
+        memset(zuc256_eia3_ooo->args.digest, 0,
+               sizeof(zuc256_eia3_ooo->args.digest));
+
         /* Init HMAC/SHA1 out-of-order fields */
         hmac_sha_1_ooo->lens[0] = 0;
         hmac_sha_1_ooo->lens[1] = 0;
@@ -1407,16 +1482,16 @@ init_mb_mgr_avx512(IMB_MGR *state)
                 MB_MGR_HMAC_SHA_512_OOO *ctx = hmac_sha_384_ooo;
 
                 ctx->ldata[j].job_in_lane = NULL;
-                ctx->ldata[j].extra_block[SHA_384_BLOCK_SIZE] = 0x80;
-                memset(ctx->ldata[j].extra_block + (SHA_384_BLOCK_SIZE + 1),
-                       0x00, SHA_384_BLOCK_SIZE + 7);
+                ctx->ldata[j].extra_block[IMB_SHA_384_BLOCK_SIZE] = 0x80;
+                memset(ctx->ldata[j].extra_block + (IMB_SHA_384_BLOCK_SIZE + 1),
+                       0x00, IMB_SHA_384_BLOCK_SIZE + 7);
                 p = ctx->ldata[j].outer_block;
                 /* special end point because this length is constant */
-                memset(p + SHA384_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
-                       SHA_384_BLOCK_SIZE -
-                       SHA384_DIGEST_SIZE_IN_BYTES  - 1 - 2);
+                memset(p + IMB_SHA384_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
+                       IMB_SHA_384_BLOCK_SIZE -
+                       IMB_SHA384_DIGEST_SIZE_IN_BYTES  - 1 - 2);
                 /* mark the end */
-                p[SHA384_DIGEST_SIZE_IN_BYTES] = 0x80;
+                p[IMB_SHA384_DIGEST_SIZE_IN_BYTES] = 0x80;
                 /* hmac outer block length always of fixed size,
                  * it is OKey length, a whole message block length, 1024 bits,
                  * with padding plus the length of the inner digest,
@@ -1424,8 +1499,8 @@ init_mb_mgr_avx512(IMB_MGR *state)
                  * The input message block needs to be converted to big endian
                  * within the sha implementation before use.
                  */
-                p[SHA_384_BLOCK_SIZE - 2] = 0x05;
-                p[SHA_384_BLOCK_SIZE - 1] = 0x80;
+                p[IMB_SHA_384_BLOCK_SIZE - 2] = 0x05;
+                p[IMB_SHA_384_BLOCK_SIZE - 1] = 0x80;
         }
 
         /* Init HMAC/SHA512 out-of-order fields */
@@ -1442,16 +1517,16 @@ init_mb_mgr_avx512(IMB_MGR *state)
                 MB_MGR_HMAC_SHA_512_OOO *ctx = hmac_sha_512_ooo;
 
                 ctx->ldata[j].job_in_lane = NULL;
-                ctx->ldata[j].extra_block[SHA_512_BLOCK_SIZE] = 0x80;
-                memset(ctx->ldata[j].extra_block + (SHA_512_BLOCK_SIZE + 1),
-                       0x00, SHA_512_BLOCK_SIZE + 7);
+                ctx->ldata[j].extra_block[IMB_SHA_512_BLOCK_SIZE] = 0x80;
+                memset(ctx->ldata[j].extra_block + (IMB_SHA_512_BLOCK_SIZE + 1),
+                       0x00, IMB_SHA_512_BLOCK_SIZE + 7);
                 p = ctx->ldata[j].outer_block;
                 /* special end point because this length is constant */
-                memset(p + SHA512_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
-                       SHA_512_BLOCK_SIZE -
-                       SHA512_DIGEST_SIZE_IN_BYTES  - 1 - 2);
+                memset(p + IMB_SHA512_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
+                       IMB_SHA_512_BLOCK_SIZE -
+                       IMB_SHA512_DIGEST_SIZE_IN_BYTES  - 1 - 2);
                 /* mark the end */
-                p[SHA512_DIGEST_SIZE_IN_BYTES] = 0x80;
+                p[IMB_SHA512_DIGEST_SIZE_IN_BYTES] = 0x80;
                 /* hmac outer block length always of fixed size,
                  * it is OKey length, a whole message block length, 1024 bits,
                  * with padding plus the length of the inner digest,
@@ -1459,8 +1534,8 @@ init_mb_mgr_avx512(IMB_MGR *state)
                  * The input message block needs to be converted to big endian
                  * within the sha implementation before use.
                  */
-                p[SHA_512_BLOCK_SIZE - 2] = 0x06;
-                p[SHA_512_BLOCK_SIZE - 1] = 0x00;
+                p[IMB_SHA_512_BLOCK_SIZE - 2] = 0x06;
+                p[IMB_SHA_512_BLOCK_SIZE - 1] = 0x00;
         }
 
         /* Init HMAC/MD5 out-of-order fields */
@@ -1574,12 +1649,14 @@ init_mb_mgr_avx512(IMB_MGR *state)
 
         /* Init AES CBC-S out-of-order fields */
         if (vaes_support) {
-                /* init 16 lanes */
-                memset(aes128_cbcs_ooo->lens, 0,
-                       sizeof(aes128_cbcs_ooo->lens));
+                /* init 12 lanes */
+                memset(aes128_cbcs_ooo->lens64, 0xFF,
+                       sizeof(aes128_cbcs_ooo->lens64));
+                memset(&aes128_cbcs_ooo->lens64[0], 0,
+                       sizeof(aes128_cbcs_ooo->lens64[0]) * 12);
                 memset(aes128_cbcs_ooo->job_in_lane, 0,
                        sizeof(aes128_cbcs_ooo->job_in_lane));
-                aes128_cbcs_ooo->unused_lanes = 0xFEDCBA9876543210;
+                aes128_cbcs_ooo->unused_lanes = 0xBA9876543210;
                 aes128_cbcs_ooo->num_lanes_inuse = 0;
 
         } else {
@@ -1697,6 +1774,26 @@ init_mb_mgr_avx512(IMB_MGR *state)
                 submit_job_aes_cntr_bit_avx512 = vaes_submit_cntr_bit_avx512;
         }
 
+        if (state->features & IMB_FEATURE_AVX512_IFMA) {
+                poly1305_mac_avx512 = poly1305_mac_fma_avx512;
+                state->chacha20_poly1305_init =
+                                        init_chacha20_poly1305_fma_avx512;
+                state->chacha20_poly1305_enc_update =
+                                        update_enc_chacha20_poly1305_fma_avx512;
+                state->chacha20_poly1305_dec_update =
+                                        update_dec_chacha20_poly1305_fma_avx512;
+                state->chacha20_poly1305_finalize =
+                                        finalize_chacha20_poly1305_fma_avx512;
+        } else {
+                state->chacha20_poly1305_init = init_chacha20_poly1305_avx512;
+                state->chacha20_poly1305_enc_update =
+                                        update_enc_chacha20_poly1305_avx512;
+                state->chacha20_poly1305_dec_update =
+                                        update_dec_chacha20_poly1305_avx512;
+                state->chacha20_poly1305_finalize =
+                                        finalize_chacha20_poly1305_avx512;
+        }
+
         if ((state->features & (IMB_FEATURE_VAES | IMB_FEATURE_VPCLMULQDQ)) ==
             (IMB_FEATURE_VAES | IMB_FEATURE_VPCLMULQDQ)) {
                 state->gcm128_enc          = aes_gcm_enc_128_vaes_avx512;
@@ -1794,6 +1891,7 @@ init_mb_mgr_avx512(IMB_MGR *state)
                 state->gmac192_finalize    = imb_aes_gmac_finalize_192_avx512;
                 state->gmac256_finalize    = imb_aes_gmac_finalize_256_avx512;
         }
+
 }
 
 #include "mb_mgr_code.h"
