@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2018-2020, Intel Corporation
+  Copyright (c) 2018-2021, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -843,25 +843,25 @@ aes_gcm_dec_var_iv_256_sse_no_aesni(const struct gcm_key_data *key_data,
 
 IMB_DLL_EXPORT void
 ghash_sse_no_aesni(struct gcm_key_data *key_data, const void *in,
-                   const uint64_t in_len, void *tag, const uint64_t tag_len);
+                   const uint64_t in_len, void *io_tag, const uint64_t tag_len);
 IMB_DLL_EXPORT void
 ghash_sse(struct gcm_key_data *key_data, const void *in,
-          const uint64_t in_len, void *tag, const uint64_t tag_len);
+          const uint64_t in_len, void *io_tag, const uint64_t tag_len);
 IMB_DLL_EXPORT void
 ghash_avx_gen2(struct gcm_key_data *key_data, const void *in,
-               const uint64_t in_len, void *tag,
+               const uint64_t in_len, void *io_tag,
                const uint64_t tag_len);
 IMB_DLL_EXPORT void
 ghash_avx_gen4(struct gcm_key_data *key_data, const void *in,
-               const uint64_t in_len, void *tag,
+               const uint64_t in_len, void *io_tag,
                const uint64_t tag_len);
 IMB_DLL_EXPORT void
 ghash_avx512(struct gcm_key_data *key_data, const void *in,
-             const uint64_t in_len, void *tag,
+             const uint64_t in_len, void *io_tag,
              const uint64_t tag_len);
 IMB_DLL_EXPORT void
 ghash_vaes_avx512(struct gcm_key_data *key_data, const void *in,
-                  const uint64_t in_len, void *tag,
+                  const uint64_t in_len, void *io_tag,
                   const uint64_t tag_len);
 
 IMB_DLL_EXPORT void
