@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2009-2020, Intel Corporation
+  Copyright (c) 2009-2021, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -297,6 +297,54 @@ IMB_DLL_LOCAL void asm_ZucInitialization_4_avx(ZucKey4_t *pKeys,
                                                ZucIv4_t *pIvs,
                                                ZucState4_t *pState);
 
+/**
+ ******************************************************************************
+ * @description
+ *      Definition of the external function that implements the initialization
+ *      stage of the ZUC-256 algorithm for 4 packets. The function will
+ *      initialize the state for 4 individual packets.
+ *
+ * @param[in] pKey                  Pointer to an array of 256-bit initial keys
+ *                                  that will be used when initializing the ZUC
+ *                                  state.
+ * @param[in] pIv                   Pointer to an array of 200-bit initial
+ *                                  vectors that will be used when initializing
+ *                                  the ZUC state (first 17 bytes are full
+ *                                  bytes, in last 8 bytes, only the lower
+ *                                  6 bits are used)
+ * @param[in,out] pState            Pointer to a ZUC state structure of type
+ *                                  @ref ZucState4_t that will be populated
+ *                                  with the initialized ZUC state.
+ * @param[in] tag_sz                Tag size (2, 4, 8 or 16), to select the
+ *                                  constants used to initialize the LFSR
+ *                                  the LFSR registers (2 is used in case of
+ *                                  encryption).
+ *
+ * @pre
+ *      None
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void asm_Zuc256Initialization_4_sse(ZucKey4_t *pKeys,
+                                                  ZucIv4_t *pIvs,
+                                                  ZucState4_t *pState,
+                                                  const unsigned tag_sz);
+
+IMB_DLL_LOCAL void asm_Zuc256Initialization_4_sse_no_aesni(ZucKey4_t *pKeys,
+                                                         ZucIv4_t *pIvs,
+                                                         ZucState4_t *pState,
+                                                         const unsigned tag_sz);
+
+IMB_DLL_LOCAL void asm_Zuc256Initialization_4_gfni_sse(ZucKey4_t *pKeys,
+                                                       ZucIv4_t *pIvs,
+                                                       ZucState4_t *pState,
+                                                       const unsigned tag_sz);
+
+IMB_DLL_LOCAL void asm_Zuc256Initialization_4_avx(ZucKey4_t *pKeys,
+                                                  ZucIv4_t *pIvs,
+                                                  ZucState4_t *pState,
+                                                  const unsigned tag_sz);
+
+
 /**
  ******************************************************************************
  * @description
@@ -322,6 +370,34 @@ IMB_DLL_LOCAL void asm_ZucInitialization_8_avx2(ZucKey8_t *pKeys,
                                                 ZucIv8_t *pIvs,
                                                 ZucState8_t *pState);
 
+/**
+ ******************************************************************************
+ * @description
+ *      Definition of the external function that implements the initialization
+ *      stage of the ZUC-256 algorithm for 8 packets. The function will
+ *      initialize the state for 8 individual packets.
+ *
+ * @param[in] pKey                  Pointer to an array of 256-bit initial keys
+ *                                  that will be used when initializing the ZUC
+ *                                  state.
+ * @param[in] pIv                   Pointer to an array of 200-bit initial
+ *                                  vectors that will be used when initializing
+ *                                  the ZUC state (first 17 bytes are full
+ *                                  bytes, in last 8 bytes, only the lower
+ *                                  6 bits are used)
+ * @param[in,out] pState            Pointer to a ZUC state structure of type
+ *                                  @ref ZucState8_t that will be populated
+ *                                  with the initialized ZUC state.
+ *
+ * @pre
+ *      None
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void asm_Zuc256Initialization_8_avx2(ZucKey8_t *pKeys,
+                                                   ZucIv8_t *pIvs,
+                                                   ZucState8_t *pState,
+                                                   const unsigned tag_sz);
+
 /**
  ******************************************************************************
  * @description
@@ -353,6 +429,41 @@ IMB_DLL_LOCAL void asm_ZucInitialization_16_gfni_avx512(ZucKey16_t *pKeys,
                                                       ZucState16_t *pState,
                                                       const uint16_t lane_mask);
 
+/**
+ ******************************************************************************
+ * @description
+ *      Definition of the external function that implements the initialization
+ *      stage of the ZUC-256 algorithm for 16 packets. The function will
+ *      initialize the state for 16 individual packets.
+ *
+ * @param[in] pKey                  Pointer to an array of 256-bit initial keys
+ *                                  that will be used when initializing the ZUC
+ *                                  state.
+ * @param[in] pIv                   Pointer to an array of 200-bit initial
+ *                                  vectors that will be used when initializing
+ *                                  the ZUC state (first 17 bytes are full
+ *                                  bytes, in last 8 bytes, only the lower
+ *                                  6 bits are used)
+ * @param[in,out] pState            Pointer to a ZUC state structure of type
+ *                                  @ref ZucState16_t that will be populated
+ *                                  with the initialized ZUC state.
+ *
+ * @pre
+ *      None
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void asm_Zuc256Initialization_16_avx512(ZucKey16_t *pKeys,
+                                                   ZucIv16_t *pIvs,
+                                                   ZucState16_t *pState,
+                                                   const uint16_t lane_mask,
+                                                   const unsigned tag_sz);
+
+IMB_DLL_LOCAL void asm_Zuc256Initialization_16_gfni_avx512(ZucKey16_t *pKeys,
+                                                      ZucIv16_t *pIvs,
+                                                      ZucState16_t *pState,
+                                                      const uint16_t lane_mask,
+                                                      const unsigned tag_sz);
+
 /**
  ******************************************************************************
  *
@@ -600,6 +711,38 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream8B_4_gfni_sse(ZucState4_t *pState,
 IMB_DLL_LOCAL void asm_ZucGenKeystream8B_4_avx(ZucState4_t *pState,
                                                uint32_t *pKeyStr[4]);
 
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate 4 bytes of
+ *      keystream for four packets in parallel.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState4_t
+ *
+ * @param[in,out] pKeyStr           Array of pointers to 4 input buffers that
+ *                                  will contain the generated keystream for
+ *                                  these 4 packets.
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization_4 to initialize the ZUC
+ *      state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void asm_ZucGenKeystream4B_4_sse(ZucState4_t *pState,
+                                               uint32_t *pKeyStr[4]);
+
+IMB_DLL_LOCAL void asm_ZucGenKeystream4B_4_sse_no_aesni(ZucState4_t *pState,
+                                                        uint32_t *pKeyStr[4]);
+
+IMB_DLL_LOCAL void asm_ZucGenKeystream4B_4_gfni_sse(ZucState4_t *pState,
+                                                    uint32_t *pKeyStr[4]);
+
+IMB_DLL_LOCAL void asm_ZucGenKeystream4B_4_avx(ZucState4_t *pState,
+                                               uint32_t *pKeyStr[4]);
+
 /**
  ******************************************************************************
  *
@@ -623,6 +766,29 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream8B_4_avx(ZucState4_t *pState,
 IMB_DLL_LOCAL void asm_ZucGenKeystream8B_8_avx2(ZucState8_t *pState,
                                                 uint32_t *pKeyStr[8]);
 
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate 4 bytes of
+ *      keystream for eight packets in parallel.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState8_t
+ *
+ * @param[in,out] pKeyStr           Array of pointers to 8 input buffers that
+ *                                  will contain the generated keystream for
+ *                                  these 8 packets.
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization_8 to initialize the ZUC
+ *      state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void asm_ZucGenKeystream4B_8_avx2(ZucState8_t *pState,
+                                                uint32_t *pKeyStr[8]);
+
 /**
  ******************************************************************************
  *
@@ -651,6 +817,33 @@ IMB_DLL_LOCAL void asm_ZucGenKeystream8B_16_gfni_avx512(ZucState16_t *pState,
                                                       uint32_t *pKeyStr[16],
                                                       const uint32_t lane_mask);
 
+/**
+ ******************************************************************************
+ *
+ * @description
+ *      Definition of the external function that implements the working
+ *      stage of the ZUC algorithm. The function will generate 4 bytes of
+ *      keystream for sixteen packets in parallel.
+ *
+ * @param[in] pState                Pointer to a ZUC state structure of type
+ *                                  @ref ZucState16_t
+ *
+ * @param[in,out] pKeyStr           Pointer to buffer to write consecutively 4
+ *                                  bytes of keystream for the 16 input buffers
+ *
+ * @pre
+ *      A successful call to @ref asm_ZucInitialization_16 to initialize the ZUC
+ *      state.
+ *
+ *****************************************************************************/
+IMB_DLL_LOCAL void asm_ZucGenKeystream4B_16_avx512(ZucState16_t *pState,
+                                                   uint32_t pKeyStr[16],
+                                                   const uint32_t lane_mask);
+
+IMB_DLL_LOCAL void asm_ZucGenKeystream4B_16_gfni_avx512(ZucState16_t *pState,
+                                                      uint32_t pKeyStr[16],
+                                                      const uint32_t lane_mask);
+
 /**
  ******************************************************************************
  *
@@ -927,6 +1120,12 @@ IMB_DLL_LOCAL uint32_t asm_Eia3RemainderAVX512_16(uint32_t *T,
                                                   uint16_t *lens,
                                                   const uint64_t commonBits);
 
+IMB_DLL_LOCAL uint32_t asm_Eia3_256_RemainderAVX512_16(uint32_t *T,
+                                                  const void * const ks,
+                                                  const void **data,
+                                                  uint16_t *lens,
+                                                  const uint64_t commonBits);
+
 IMB_DLL_LOCAL
 void zuc_eia3_4_buffer_job_gfni_sse(const void * const pKey[4],
                                     const void * const pIv[4],
@@ -951,6 +1150,30 @@ void zuc_eia3_4_buffer_job_sse_no_aesni(const void * const pKey[4],
                                         const uint16_t lengthInBits[4],
                                         const void * const job_in_lane[4]);
 
+IMB_DLL_LOCAL
+void zuc256_eia3_4_buffer_job_gfni_sse(const void * const pKey[4],
+                                       const void * const pIv[4],
+                                       const void * const pBufferIn[4],
+                                       uint32_t *pMacI[4],
+                                       const uint16_t lengthInBits[4],
+                                       const void * const job_in_lane[4]);
+
+IMB_DLL_LOCAL
+void zuc256_eia3_4_buffer_job_no_gfni_sse(const void * const pKey[4],
+                                          const void * const pIv[4],
+                                          const void * const pBufferIn[4],
+                                          uint32_t *pMacI[4],
+                                          const uint16_t lengthInBits[4],
+                                          const void * const job_in_lane[4]);
+
+IMB_DLL_LOCAL
+void zuc256_eia3_4_buffer_job_sse_no_aesni(const void * const pKey[4],
+                                           const void * const pIv[4],
+                                           const void * const pBufferIn[4],
+                                           uint32_t *pMacI[4],
+                                           const uint16_t lengthInBits[4],
+                                           const void * const job_in_lane[4]);
+
 IMB_DLL_LOCAL
 void zuc_eia3_4_buffer_job_avx(const void * const pKey[4],
                                const void * const pIv[4],
@@ -959,6 +1182,14 @@ void zuc_eia3_4_buffer_job_avx(const void * const pKey[4],
                                const uint16_t lengthInBits[4],
                                const void * const job_in_lane[4]);
 
+IMB_DLL_LOCAL
+void zuc256_eia3_4_buffer_job_avx(const void * const pKey[4],
+                               const void * const pIv[4],
+                               const void * const pBufferIn[4],
+                               uint32_t *pMacI[4],
+                               const uint16_t lengthInBits[4],
+                               const void * const job_in_lane[4]);
+
 IMB_DLL_LOCAL
 void zuc_eia3_8_buffer_job_avx2(const void * const pKey[8],
                                 const void * const pIv[8],
@@ -967,12 +1198,26 @@ void zuc_eia3_8_buffer_job_avx2(const void * const pKey[8],
                                 const uint16_t lengthInBits[8],
                                 const void * const job_in_lane[8]);
 
+IMB_DLL_LOCAL
+void zuc256_eia3_8_buffer_job_avx2(const void * const pKey[8],
+                                   const void * const pIv[8],
+                                   const void * const pBufferIn[8],
+                                   uint32_t *pMacI[8],
+                                   const uint16_t lengthInBits[8],
+                                   const void * const job_in_lane[8]);
+
 IMB_DLL_LOCAL
 void zuc_eia3_16_buffer_job_no_gfni_avx512(MB_MGR_ZUC_OOO *ooo);
 
 IMB_DLL_LOCAL
 void zuc_eia3_16_buffer_job_gfni_avx512(MB_MGR_ZUC_OOO *ooo);
 
+IMB_DLL_LOCAL
+void zuc256_eia3_16_buffer_job_no_gfni_avx512(MB_MGR_ZUC_OOO *ooo);
+
+IMB_DLL_LOCAL
+void zuc256_eia3_16_buffer_job_gfni_avx512(MB_MGR_ZUC_OOO *ooo);
+
 /* the s-boxes */
 extern const uint8_t S0[256];
 extern const uint8_t S1[256];
