@@ -1,5 +1,5 @@
 /*******************************************************************************
-  Copyright (c) 2018-2020, Intel Corporation
+  Copyright (c) 2018-2021, Intel Corporation
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
@@ -109,13 +109,21 @@ IMB_JOB *submit_job_aes_cntr_sse_no_aesni(IMB_JOB *job);
 
 IMB_JOB *submit_job_aes_cntr_bit_sse_no_aesni(IMB_JOB *job);
 
-JOB_AES_HMAC *submit_job_zuc_eea3_sse_no_aesni(MB_MGR_ZUC_OOO *state,
+IMB_JOB *submit_job_zuc_eea3_sse_no_aesni(MB_MGR_ZUC_OOO *state,
                                                IMB_JOB *job);
-JOB_AES_HMAC *flush_job_zuc_eea3_sse_no_aesni(MB_MGR_ZUC_OOO *state);
+IMB_JOB *flush_job_zuc_eea3_sse_no_aesni(MB_MGR_ZUC_OOO *state);
 
-JOB_AES_HMAC *submit_job_zuc_eia3_sse_no_aesni(MB_MGR_ZUC_OOO *state,
+IMB_JOB *submit_job_zuc256_eea3_sse_no_aesni(MB_MGR_ZUC_OOO *state,
                                                IMB_JOB *job);
-JOB_AES_HMAC *flush_job_zuc_eia3_sse_no_aesni(MB_MGR_ZUC_OOO *state);
+IMB_JOB *flush_job_zuc256_eea3_sse_no_aesni(MB_MGR_ZUC_OOO *state);
+
+IMB_JOB *submit_job_zuc_eia3_sse_no_aesni(MB_MGR_ZUC_OOO *state,
+                                               IMB_JOB *job);
+IMB_JOB *flush_job_zuc_eia3_sse_no_aesni(MB_MGR_ZUC_OOO *state);
+
+IMB_JOB *submit_job_zuc256_eia3_sse_no_aesni(MB_MGR_ZUC_OOO *state,
+                                             IMB_JOB *job);
+IMB_JOB *flush_job_zuc256_eia3_sse_no_aesni(MB_MGR_ZUC_OOO *state);
 
 uint32_t hec_32_sse_no_aesni(const uint8_t *in);
 uint64_t hec_64_sse_no_aesni(const uint8_t *in);
@@ -126,6 +134,11 @@ IMB_JOB *flush_job_aes128_cbcs_1_9_enc_sse_no_aesni(MB_MGR_AES_OOO *state);
 
 IMB_JOB *submit_job_chacha20_enc_dec_sse(IMB_JOB *job);
 
+void *poly1305_mac_scalar(IMB_JOB *job);
+
+IMB_JOB *snow_v_sse_no_aesni(IMB_JOB *job);
+IMB_JOB *snow_v_aead_init_sse_no_aesni(IMB_JOB *job);
+
 #define SAVE_XMMS               save_xmms
 #define RESTORE_XMMS            restore_xmms
 
@@ -172,6 +185,10 @@ IMB_JOB *submit_job_chacha20_enc_dec_sse(IMB_JOB *job);
 #define FLUSH_JOB_ZUC_EEA3    flush_job_zuc_eea3_sse_no_aesni
 #define SUBMIT_JOB_ZUC_EIA3   submit_job_zuc_eia3_sse_no_aesni
 #define FLUSH_JOB_ZUC_EIA3    flush_job_zuc_eia3_sse_no_aesni
+#define SUBMIT_JOB_ZUC256_EEA3   submit_job_zuc256_eea3_sse_no_aesni
+#define FLUSH_JOB_ZUC256_EEA3    flush_job_zuc256_eea3_sse_no_aesni
+#define SUBMIT_JOB_ZUC256_EIA3   submit_job_zuc256_eia3_sse_no_aesni
+#define FLUSH_JOB_ZUC256_EIA3    flush_job_zuc256_eia3_sse_no_aesni
 
 #define AES_CBC_DEC_128       aes_cbc_dec_128_sse_no_aesni
 #define AES_CBC_DEC_192       aes_cbc_dec_192_sse_no_aesni
@@ -211,9 +228,7 @@ IMB_JOB *submit_job_chacha20_enc_dec_sse(IMB_JOB *job);
 #define AES_GCM_ENC_IV_256   aes_gcm_enc_var_iv_256_sse_no_aesni
 
 #define SUBMIT_JOB_AES_GCM_DEC submit_job_aes_gcm_dec_sse_no_aesni
-#define FLUSH_JOB_AES_GCM_DEC  flush_job_aes_gcm_dec_sse_no_aesni
 #define SUBMIT_JOB_AES_GCM_ENC submit_job_aes_gcm_enc_sse_no_aesni
-#define FLUSH_JOB_AES_GCM_ENC  flush_job_aes_gcm_enc_sse_no_aesni
 
 /* ====================================================================== */
 
@@ -269,6 +284,11 @@ void aes128_cbc_mac_x4_no_aesni(AES_ARGS *args, uint64_t len);
         aes_cbcs_1_9_dec_128_sse_no_aesni
 #define SUBMIT_JOB_CHACHA20_ENC_DEC submit_job_chacha20_enc_dec_sse
 #define SUBMIT_JOB_CHACHA20_POLY1305 aead_chacha20_poly1305_sse
+#define SUBMIT_JOB_CHACHA20_POLY1305_SGL aead_chacha20_poly1305_sgl_sse
+#define POLY1305_MAC poly1305_mac_scalar
+
+#define SUBMIT_JOB_SNOW_V snow_v_sse_no_aesni
+#define SUBMIT_JOB_SNOW_V_AEAD snow_v_aead_init_sse_no_aesni
 
 /* ====================================================================== */
 
@@ -338,18 +358,10 @@ submit_job_aes_gcm_dec_sse_no_aesni(IMB_MGR *state, IMB_JOB *job)
                                    job->auth_tag_output_len_in_bytes);
         }
 
-        job->status = STS_COMPLETED;
+        job->status = IMB_STATUS_COMPLETED;
         return job;
 }
 
-static IMB_JOB *
-flush_job_aes_gcm_dec_sse_no_aesni(IMB_MGR *state, IMB_JOB *job)
-{
-        (void) state;
-        (void) job;
-        return NULL;
-}
-
 static IMB_JOB *
 submit_job_aes_gcm_enc_sse_no_aesni(IMB_MGR *state, IMB_JOB *job)
 {
@@ -391,18 +403,10 @@ submit_job_aes_gcm_enc_sse_no_aesni(IMB_MGR *state, IMB_JOB *job)
                                    job->auth_tag_output_len_in_bytes);
         }
 
-        job->status = STS_COMPLETED;
+        job->status = IMB_STATUS_COMPLETED;
         return job;
 }
 
-static IMB_JOB *
-flush_job_aes_gcm_enc_sse_no_aesni(IMB_MGR *state, IMB_JOB *job)
-{
-        (void) state;
-        (void) job;
-        return NULL;
-}
-
 IMB_DLL_LOCAL IMB_JOB *
 submit_job_aes_cntr_sse_no_aesni(IMB_JOB *job)
 {
@@ -428,7 +432,7 @@ submit_job_aes_cntr_sse_no_aesni(IMB_JOB *job)
                              job->msg_len_to_cipher_in_bytes,
                              job->iv_len_in_bytes);
 
-        job->status |= STS_COMPLETED_AES;
+        job->status |= IMB_STATUS_COMPLETED_CIPHER;
         return job;
 }
 
@@ -459,7 +463,7 @@ submit_job_aes_cntr_bit_sse_no_aesni(IMB_JOB *job)
                                               job->msg_len_to_cipher_in_bits,
                                               job->iv_len_in_bytes);
 
-        job->status |= STS_COMPLETED_AES;
+        job->status |= IMB_STATUS_COMPLETED_CIPHER;
         return job;
 }
 
@@ -498,8 +502,10 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
         MB_MGR_CCM_OOO *aes_ccm_ooo = state->aes_ccm_ooo;
         MB_MGR_CMAC_OOO *aes_cmac_ooo = state->aes_cmac_ooo;
         MB_MGR_ZUC_OOO *zuc_eea3_ooo = state->zuc_eea3_ooo;
+        MB_MGR_ZUC_OOO *zuc256_eea3_ooo = state->zuc256_eea3_ooo;
         MB_MGR_ZUC_OOO *zuc_eia3_ooo = state->zuc_eia3_ooo;
         MB_MGR_AES_OOO *aes128_cbcs_ooo = state->aes128_cbcs_ooo;
+        MB_MGR_ZUC_OOO *zuc256_eia3_ooo = state->zuc256_eia3_ooo;
 
         /* reset error status */
         imb_set_errno(state, 0);
@@ -595,6 +601,28 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
         zuc_eia3_ooo->init_not_done = 0;
         zuc_eia3_ooo->unused_lane_bitmask = 0x0f;
 
+        memset(zuc256_eea3_ooo->lens, 0,
+               sizeof(zuc256_eea3_ooo->lens));
+        memset(zuc256_eea3_ooo->job_in_lane, 0,
+               sizeof(zuc256_eea3_ooo->job_in_lane));
+        zuc256_eea3_ooo->unused_lanes = 0xFF03020100;
+        zuc256_eea3_ooo->num_lanes_inuse = 0;
+        memset(&zuc256_eea3_ooo->state, 0,
+               sizeof(zuc256_eea3_ooo->state));
+        zuc256_eea3_ooo->init_not_done = 0;
+        zuc256_eea3_ooo->unused_lane_bitmask = 0x0f;
+
+        memset(zuc256_eia3_ooo->lens, 0xFF,
+               sizeof(zuc256_eia3_ooo->lens));
+        memset(zuc256_eia3_ooo->job_in_lane, 0,
+               sizeof(zuc256_eia3_ooo->job_in_lane));
+        zuc256_eia3_ooo->unused_lanes = 0xFF03020100;
+        zuc256_eia3_ooo->num_lanes_inuse = 0;
+        memset(&zuc256_eia3_ooo->state, 0,
+               sizeof(zuc256_eia3_ooo->state));
+        zuc256_eia3_ooo->init_not_done = 0;
+        zuc256_eia3_ooo->unused_lane_bitmask = 0x0f;
+
         /* Init HMAC/SHA1 out-of-order fields */
         hmac_sha_1_ooo->lens[0] = 0;
         hmac_sha_1_ooo->lens[1] = 0;
@@ -686,16 +714,16 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
                 MB_MGR_HMAC_SHA_512_OOO *ctx = hmac_sha_384_ooo;
 
                 ctx->ldata[j].job_in_lane = NULL;
-                ctx->ldata[j].extra_block[SHA_384_BLOCK_SIZE] = 0x80;
-                memset(ctx->ldata[j].extra_block + (SHA_384_BLOCK_SIZE + 1),
-                       0x00, SHA_384_BLOCK_SIZE + 7);
+                ctx->ldata[j].extra_block[IMB_SHA_384_BLOCK_SIZE] = 0x80;
+                memset(ctx->ldata[j].extra_block + (IMB_SHA_384_BLOCK_SIZE + 1),
+                       0x00, IMB_SHA_384_BLOCK_SIZE + 7);
 
                 p = ctx->ldata[j].outer_block;
-                memset(p + SHA384_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
+                memset(p + IMB_SHA384_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
                        /* special end point because this length is constant */
-                       SHA_384_BLOCK_SIZE -
-                       SHA384_DIGEST_SIZE_IN_BYTES - 1 - 2);
-                p[SHA384_DIGEST_SIZE_IN_BYTES] = 0x80; /* mark the end */
+                       IMB_SHA_384_BLOCK_SIZE -
+                       IMB_SHA384_DIGEST_SIZE_IN_BYTES - 1 - 2);
+                p[IMB_SHA384_DIGEST_SIZE_IN_BYTES] = 0x80; /* mark the end */
                 /*
                  * hmac outer block length always of fixed size, it is OKey
                  * length, a whole message block length, 1024 bits, with padding
@@ -704,8 +732,8 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
                  * converted to big endian within the sha implementation
                  * before use.
                  */
-                p[SHA_384_BLOCK_SIZE - 2] = 0x05;
-                p[SHA_384_BLOCK_SIZE - 1] = 0x80;
+                p[IMB_SHA_384_BLOCK_SIZE - 2] = 0x05;
+                p[IMB_SHA_384_BLOCK_SIZE - 1] = 0x80;
         }
 
         /* Init HMAC/SHA512 out-of-order fields */
@@ -722,16 +750,16 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
                 MB_MGR_HMAC_SHA_512_OOO *ctx = hmac_sha_512_ooo;
 
                 ctx->ldata[j].job_in_lane = NULL;
-                ctx->ldata[j].extra_block[SHA_512_BLOCK_SIZE] = 0x80;
-                memset(ctx->ldata[j].extra_block + (SHA_512_BLOCK_SIZE + 1),
-                       0x00, SHA_512_BLOCK_SIZE + 7);
+                ctx->ldata[j].extra_block[IMB_SHA_512_BLOCK_SIZE] = 0x80;
+                memset(ctx->ldata[j].extra_block + (IMB_SHA_512_BLOCK_SIZE + 1),
+                       0x00, IMB_SHA_512_BLOCK_SIZE + 7);
 
                 p = ctx->ldata[j].outer_block;
-                memset(p + SHA512_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
+                memset(p + IMB_SHA512_DIGEST_SIZE_IN_BYTES  + 1, 0x00,
                        /* special end point because this length is constant */
-                       SHA_512_BLOCK_SIZE -
-                       SHA512_DIGEST_SIZE_IN_BYTES  - 1 - 2);
-                p[SHA512_DIGEST_SIZE_IN_BYTES] = 0x80; /* mark the end */
+                       IMB_SHA_512_BLOCK_SIZE -
+                       IMB_SHA512_DIGEST_SIZE_IN_BYTES  - 1 - 2);
+                p[IMB_SHA512_DIGEST_SIZE_IN_BYTES] = 0x80; /* mark the end */
                 /*
                  * hmac outer block length always of fixed size, it is OKey
                  * length, a whole message block length, 1024 bits, with padding
@@ -740,8 +768,8 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
                  * converted to big endian within the sha implementation
                  * before use.
                  */
-                p[SHA_512_BLOCK_SIZE - 2] = 0x06;
-                p[SHA_512_BLOCK_SIZE - 1] = 0x00;
+                p[IMB_SHA_512_BLOCK_SIZE - 2] = 0x06;
+                p[IMB_SHA_512_BLOCK_SIZE - 1] = 0x00;
         }
 
         /* Init HMAC/MD5 out-of-order fields */
@@ -940,6 +968,11 @@ init_mb_mgr_sse_no_aesni(IMB_MGR *state)
         state->crc6_iuup_header    = crc6_iuup_header_sse_no_aesni;
         state->crc32_wimax_ofdma_data = crc32_wimax_ofdma_data_sse_no_aesni;
         state->crc8_wimax_ofdma_hcs = crc8_wimax_ofdma_hcs_sse_no_aesni;
+
+        state->chacha20_poly1305_init = init_chacha20_poly1305_sse;
+        state->chacha20_poly1305_enc_update = update_enc_chacha20_poly1305_sse;
+        state->chacha20_poly1305_dec_update = update_dec_chacha20_poly1305_sse;
+        state->chacha20_poly1305_finalize = finalize_chacha20_poly1305_sse;
 }
 
 #include "mb_mgr_code.h"
