@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2018-2020, Intel Corporation
+ Copyright (c) 2018-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -1828,7 +1828,7 @@ aes_job_ok(const struct IMB_JOB *job,
 {
         const int num = (const int)((uint64_t)job->user_data2);
 
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 printf("%d error status:%d, job %d",
                        __LINE__, job->status, num);
                 return 0;
@@ -1861,7 +1861,7 @@ test_aes_many(struct IMB_MGR *mb_mgr,
               const unsigned text_len,
               const int dir,
               const int order,
-              const JOB_CIPHER_MODE cipher,
+              const IMB_CIPHER_MODE cipher,
               const int in_place,
               const int key_len,
               const int num_jobs)
@@ -1889,7 +1889,7 @@ test_aes_many(struct IMB_MGR *mb_mgr,
         }
 
         /* flush the scheduler */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
@@ -1954,7 +1954,7 @@ test_aes_many(struct IMB_MGR *mb_mgr,
         ret = 0;
 
  end:
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL) {
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL) {
                 err = imb_get_errno(mb_mgr);
                 if (err != 0) {
                         printf("Error: %s!\n", imb_get_strerror(err));
@@ -1979,7 +1979,7 @@ test_aes_vectors(struct IMB_MGR *mb_mgr,
                  struct test_suite_context *ctx256,
                  const int vec_cnt,
                  const struct aes_vector *vec_tab, const char *banner,
-                 const JOB_CIPHER_MODE cipher, const int num_jobs)
+                 const IMB_CIPHER_MODE cipher, const int num_jobs)
 {
 	int vect;
         DECLARE_ALIGNED(uint32_t enc_keys[15*4], 16);
@@ -2080,7 +2080,7 @@ docrc_job_ok(const struct IMB_JOB *job,
         const int num = (const int)((uint64_t)job->user_data2);
         const unsigned text_len = (unsigned) p_vec->frame_len;
 
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 printf("%d error status:%d, job %d",
                        __LINE__, job->status, num);
                 return 0;
@@ -2179,7 +2179,7 @@ test_docrc_many(struct IMB_MGR *mb_mgr,
         }
 
         /* flush the scheduler */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
@@ -2235,7 +2235,7 @@ test_docrc_many(struct IMB_MGR *mb_mgr,
         ret = 0;
 
  end:
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
  end_alloc:
 
