@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2018-2020, Intel Corporation
+ Copyright (c) 2018-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -54,7 +54,8 @@ enum {
       TEST_CIPH_ENC_KEY_NULL,
       TEST_CIPH_DEC_KEY_NULL,
       TEST_CIPH_MSG_LEN_ZERO,
-      TEST_CIPH_MSG_LEN_GT_MAX
+      TEST_CIPH_MSG_LEN_GT_MAX,
+      TEST_CIPH_NEXT_IV_NULL,
 };
 
 /*
@@ -102,10 +103,10 @@ test_job_api(struct IMB_MGR *mb_mgr)
         }
         printf(".");
 
-        if (job_next->status != STS_INVALID_ARGS) {
+        if (job_next->status != IMB_STATUS_INVALID_ARGS) {
                 /* Invalid job is returned, and status should be INVALID_ARGS */
                 printf("%s: test %d, unexpected job->status != "
-                       "STS_INVALID_ARGS\n", __func__, TEST_INVALID_JOB);
+                       "IMB_STATUS_INVALID_ARGS\n", __func__, TEST_INVALID_JOB);
                 return 1;
         }
 	printf(".");
@@ -143,7 +144,7 @@ test_job_api(struct IMB_MGR *mb_mgr)
 	printf(".");
 
         /* clean up */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
 	printf("\n");
@@ -164,10 +165,12 @@ static int dummy_cipher_hash_func(struct IMB_JOB *job)
  */
 static void
 fill_in_job(struct IMB_JOB *job,
-            const JOB_CIPHER_MODE cipher_mode,
-            const JOB_CIPHER_DIRECTION cipher_direction,
-            const JOB_HASH_ALG hash_alg,
-            const JOB_CHAIN_ORDER chain_order)
+            const IMB_CIPHER_MODE cipher_mode,
+            const IMB_CIPHER_DIRECTION cipher_direction,
+            const IMB_HASH_ALG hash_alg,
+            const IMB_CHAIN_ORDER chain_order,
+            struct chacha20_poly1305_context_data *chacha_ctx,
+            struct gcm_context_data *gcm_ctx)
 {
         const uint64_t tag_len_tab[] = {
                 0,  /* INVALID selection */
@@ -200,6 +203,22 @@ fill_in_job(struct IMB_JOB *job,
                 16, /* IMB_AUTH_AES_CMAC_256 */
                 16, /* IMB_AUTH_POLY1305 */
                 16, /* IMB_AUTH_CHACHA20_POLY1305 */
+                16, /* IMB_AUTH_CHACHA20_POLY1305_SGL */
+                4,  /* IMB_AUTH_ZUC256_EIA3_BITLEN */
+                16, /* IMB_AUTH_SNOW_V_AEAD */
+                16, /* IMB_AUTH_AES_GCM_SGL */
+                4,  /* IMB_AUTH_CRC32_ETHERNET_FCS */
+                4,  /* IMB_AUTH_CRC32_SCTP */
+                4,  /* IMB_AUTH_CRC32_WIMAX_OFDMA_DATA */
+                4,  /* IMB_AUTH_CRC24_LTE_A */
+                4,  /* IMB_AUTH_CRC24_LTE_B */
+                4,  /* IMB_AUTH_CRC16_X25 */
+                4,  /* IMB_AUTH_CRC16_FP_DATA */
+                4,  /* IMB_AUTH_CRC11_FP_HEADER */
+                4,  /* IMB_AUTH_CRC10_IUUP_DATA */
+                4,  /* IMB_AUTH_CRC8_WIMAX_OFDMA_HCS */
+                4,  /* IMB_AUTH_CRC7_FP_HEADER */
+                4,  /* IMB_AUTH_CRC6_IUUP_HEADER */
         };
         static DECLARE_ALIGNED(uint8_t dust_bin[2048], 64);
         const uint64_t msg_len_to_cipher = 32;
@@ -297,6 +316,26 @@ fill_in_job(struct IMB_JOB *job,
                 job->key_len_in_bytes = UINT64_C(32);
                 job->iv_len_in_bytes = 12;
                 break;
+        case IMB_CIPHER_CHACHA20_POLY1305_SGL:
+                job->hash_alg = IMB_AUTH_CHACHA20_POLY1305_SGL;
+                job->key_len_in_bytes = UINT64_C(32);
+                job->iv_len_in_bytes = 12;
+                break;
+        case IMB_CIPHER_SNOW_V:
+                job->hash_alg = IMB_AUTH_NULL;
+                job->key_len_in_bytes = UINT64_C(32);
+                job->iv_len_in_bytes = 16;
+                break;
+        case IMB_CIPHER_SNOW_V_AEAD:
+                job->hash_alg = IMB_AUTH_SNOW_V_AEAD;
+                job->key_len_in_bytes = UINT64_C(32);
+                job->iv_len_in_bytes = 16;
+                break;
+        case IMB_CIPHER_GCM_SGL:
+                job->hash_alg = IMB_AUTH_GCM_SGL;
+                job->key_len_in_bytes = UINT64_C(16);
+                job->iv_len_in_bytes = UINT64_C(12);
+                break;
         default:
                 break;
         }
@@ -313,6 +352,18 @@ fill_in_job(struct IMB_JOB *job,
         case IMB_AUTH_SHA_256:
         case IMB_AUTH_SHA_384:
         case IMB_AUTH_SHA_512:
+        case IMB_AUTH_CRC32_ETHERNET_FCS:
+        case IMB_AUTH_CRC32_SCTP:
+        case IMB_AUTH_CRC32_WIMAX_OFDMA_DATA:
+        case IMB_AUTH_CRC24_LTE_A:
+        case IMB_AUTH_CRC24_LTE_B:
+        case IMB_AUTH_CRC16_X25:
+        case IMB_AUTH_CRC16_FP_DATA:
+        case IMB_AUTH_CRC11_FP_HEADER:
+        case IMB_AUTH_CRC10_IUUP_DATA:
+        case IMB_AUTH_CRC8_WIMAX_OFDMA_HCS:
+        case IMB_AUTH_CRC7_FP_HEADER:
+        case IMB_AUTH_CRC6_IUUP_HEADER:
         case IMB_AUTH_NULL:
                 break;
         case IMB_AUTH_AES_XCBC:
@@ -352,6 +403,8 @@ fill_in_job(struct IMB_JOB *job,
         case IMB_AUTH_PON_CRC_BIP:
                 job->msg_len_to_hash_in_bytes = 8;
                 job->auth_tag_output_len_in_bytes = 8;
+                job->hash_start_src_offset_in_bytes = 0;
+                job->cipher_start_src_offset_in_bytes = 8;
                 /* set required cipher mode fields */
                 job->cipher_mode = IMB_CIPHER_PON_AES_CNTR;
                 job->dst = dust_bin + 8;
@@ -360,6 +413,7 @@ fill_in_job(struct IMB_JOB *job,
                 job->iv_len_in_bytes = 16;
                 break;
         case IMB_AUTH_ZUC_EIA3_BITLEN:
+        case IMB_AUTH_ZUC256_EIA3_BITLEN:
                 job->u.ZUC_EIA3._key = dust_bin;
                 job->u.ZUC_EIA3._iv = dust_bin;
                 job->auth_tag_output_len_in_bytes = 4;
@@ -407,6 +461,30 @@ fill_in_job(struct IMB_JOB *job,
                 job->u.CHACHA20_POLY1305.aad_len_in_bytes = 12;
                 job->auth_tag_output_len_in_bytes = 16;
                 break;
+        case IMB_AUTH_CHACHA20_POLY1305_SGL:
+                job->cipher_mode = IMB_CIPHER_CHACHA20_POLY1305_SGL;
+                job->key_len_in_bytes = UINT64_C(32);
+                job->iv_len_in_bytes = 12;
+                job->u.CHACHA20_POLY1305.aad = dust_bin;
+                job->u.CHACHA20_POLY1305.aad_len_in_bytes = 12;
+                job->auth_tag_output_len_in_bytes = 16;
+                job->u.CHACHA20_POLY1305.ctx = chacha_ctx;
+                break;
+        case IMB_AUTH_GCM_SGL:
+                job->u.GCM.ctx = gcm_ctx;
+                job->u.GCM.aad = dust_bin;
+                job->u.GCM.aad_len_in_bytes = 16;
+                /* set required cipher mode fields */
+                job->cipher_mode = IMB_CIPHER_GCM_SGL;
+                job->key_len_in_bytes = UINT64_C(16);
+                job->iv_len_in_bytes = UINT64_C(12);
+                break;
+        case IMB_AUTH_SNOW_V_AEAD:
+                job->cipher_mode = IMB_CIPHER_SNOW_V_AEAD;
+                job->key_len_in_bytes = UINT64_C(32);
+                job->iv_len_in_bytes = 16;
+                job->auth_tag_output_len_in_bytes = 16;
+                break;
         default:
                 break;
         }
@@ -494,10 +572,10 @@ is_submit_invalid(struct IMB_MGR *mb_mgr, const struct IMB_JOB *job,
                 }
         }
 
-        if (job_ret->status != STS_INVALID_ARGS) {
+        if (job_ret->status != IMB_STATUS_INVALID_ARGS) {
                 printf("%s : test %d, hash_alg %d, chain_order %d, "
                        "cipher_dir %d, cipher_mode %d : "
-                       "unexpected job->status %d != STS_INVALID_ARGS\n",
+                       "unexpected job->status %d != IMB_STATUS_INVALID_ARGS\n",
                        __func__, test_num, (int) job_ret->hash_alg,
                        (int) job_ret->chain_order,
                        (int) job_ret->cipher_direction,
@@ -508,23 +586,50 @@ is_submit_invalid(struct IMB_MGR *mb_mgr, const struct IMB_JOB *job,
         return 1;
 }
 
+/*
+ * @brief Checks for AEAD algorithms
+ */
+static int
+check_aead(IMB_HASH_ALG hash, IMB_CIPHER_MODE cipher)
+{
+        if (hash == IMB_AUTH_CHACHA20_POLY1305 ||
+            hash == IMB_AUTH_CHACHA20_POLY1305_SGL ||
+            hash == IMB_AUTH_GCM_SGL ||
+            hash == IMB_AUTH_AES_GMAC ||
+            hash == IMB_AUTH_AES_CCM ||
+            hash == IMB_AUTH_SNOW_V_AEAD ||
+            hash == IMB_AUTH_PON_CRC_BIP)
+                return 1;
+
+        if (cipher == IMB_CIPHER_CHACHA20_POLY1305 ||
+            cipher == IMB_CIPHER_CHACHA20_POLY1305_SGL ||
+            cipher == IMB_CIPHER_GCM_SGL ||
+            cipher == IMB_CIPHER_GCM ||
+            cipher == IMB_CIPHER_CCM ||
+            cipher == IMB_CIPHER_SNOW_V_AEAD ||
+            cipher == IMB_CIPHER_PON_AES_CNTR)
+                return 1;
+        return 0;
+}
+
 /*
  * @brief Tests invalid settings for MAC modes
  */
 static int
 test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
 {
-        JOB_HASH_ALG hash;
-        JOB_CIPHER_DIRECTION dir;
-        const JOB_CIPHER_MODE cipher = IMB_CIPHER_NULL;
-        JOB_CHAIN_ORDER order;
+        IMB_HASH_ALG hash;
+        IMB_CIPHER_DIRECTION dir;
+        const IMB_CIPHER_MODE cipher = IMB_CIPHER_NULL;
+        IMB_CHAIN_ORDER order;
         struct IMB_JOB template_job;
-        struct IMB_JOB *job;
+        struct chacha20_poly1305_context_data chacha_ctx;
+        struct gcm_context_data gcm_ctx;
 
 	printf("Invalid JOB MAC arguments test:\n");
 
         /* prep */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         /*
@@ -539,8 +644,15 @@ test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
                                     hash == IMB_AUTH_CUSTOM)
                                         continue;
 
+                                /*
+                                 * Skip hash algorithms belonging to AEAD
+                                 * algorithms, as the test is for authentication
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
+                                        continue;
+
                                 fill_in_job(&template_job, cipher, dir,
-                                            hash, order);
+                                            hash, order, &chacha_ctx, &gcm_ctx);
                                 template_job.src = NULL;
                                 if (!is_submit_invalid(mb_mgr,
                                                        &template_job,
@@ -562,9 +674,19 @@ test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
                                     hash == IMB_AUTH_CUSTOM)
                                         continue;
 
+                                /*
+                                 * Skip hash algorithms belonging to AEAD
+                                 * algorithms, as the test is for authentication
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
+                                        continue;
+
                                 fill_in_job(&template_job, cipher, dir,
-                                            hash, order);
+                                            hash, order, &chacha_ctx, &gcm_ctx);
                                 template_job.auth_tag_output = NULL;
+                                if (hash == IMB_AUTH_GCM_SGL)
+                                        template_job.sgl_state =
+                                                IMB_SGL_COMPLETE;
                                 if (!is_submit_invalid(mb_mgr, &template_job,
                                                  TEST_AUTH_AUTH_TAG_OUTPUT_NULL,
                                                  IMB_ERR_JOB_NULL_AUTH))
@@ -584,9 +706,19 @@ test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
                                     hash == IMB_AUTH_CUSTOM)
                                         continue;
 
+                                /*
+                                 * Skip hash algorithms belonging to AEAD
+                                 * algorithms, as the test is for authentication
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
+                                        continue;
+
                                 fill_in_job(&template_job, cipher, dir,
-                                            hash, order);
+                                            hash, order, &chacha_ctx, &gcm_ctx);
                                 template_job.auth_tag_output_len_in_bytes = 0;
+                                if (hash == IMB_AUTH_GCM_SGL)
+                                        template_job.sgl_state =
+                                                IMB_SGL_COMPLETE;
                                 if (!is_submit_invalid(mb_mgr, &template_job,
                                                   TEST_AUTH_TAG_OUTPUT_LEN_ZERO,
                                                   IMB_ERR_JOB_AUTH_TAG_LEN))
@@ -610,20 +742,35 @@ test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
                                     hash == IMB_AUTH_AES_GMAC_128 ||
                                     hash == IMB_AUTH_AES_GMAC_192 ||
                                     hash == IMB_AUTH_AES_GMAC_256 ||
+                                    hash == IMB_AUTH_SNOW_V_AEAD ||
+                                    hash == IMB_AUTH_CRC32_ETHERNET_FCS ||
+                                    hash == IMB_AUTH_CRC32_SCTP ||
+                                    hash == IMB_AUTH_CRC32_WIMAX_OFDMA_DATA ||
+                                    hash == IMB_AUTH_CRC24_LTE_A ||
+                                    hash == IMB_AUTH_CRC24_LTE_B ||
+                                    hash == IMB_AUTH_CRC16_X25 ||
+                                    hash == IMB_AUTH_CRC16_FP_DATA ||
+                                    hash == IMB_AUTH_CRC11_FP_HEADER ||
+                                    hash == IMB_AUTH_CRC10_IUUP_DATA ||
+                                    hash == IMB_AUTH_CRC8_WIMAX_OFDMA_HCS ||
+                                    hash == IMB_AUTH_CRC7_FP_HEADER ||
+                                    hash == IMB_AUTH_CRC6_IUUP_HEADER ||
                                     hash == IMB_AUTH_POLY1305)
                                         continue;
 
-                                /* skip disallowed combos */
-                                if ((hash == IMB_AUTH_CHACHA20_POLY1305 &&
-                                     cipher != IMB_CIPHER_CHACHA20_POLY1305) ||
-                                    (hash != IMB_AUTH_CHACHA20_POLY1305 &&
-                                     cipher == IMB_CIPHER_CHACHA20_POLY1305))
+                                /*
+                                 * Skip hash algorithms belonging to AEAD
+                                 * algorithms, as the test is for authentication
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
                                         continue;
 
                                 fill_in_job(&template_job, cipher, dir,
-                                            hash, order);
+                                            hash, order, &chacha_ctx, &gcm_ctx);
+
                                 switch (hash) {
                                 case IMB_AUTH_ZUC_EIA3_BITLEN:
+                                case IMB_AUTH_ZUC256_EIA3_BITLEN:
                                         /* (2^32) - 32 is max */
                                         template_job.msg_len_to_hash_in_bytes =
                                                 ((1ULL << 32) - 31);
@@ -639,6 +786,7 @@ test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
                                                 (20008 / 8); /* 2501 bytes */
                                         break;
                                 case IMB_AUTH_CHACHA20_POLY1305:
+                                case IMB_AUTH_CHACHA20_POLY1305_SGL:
                                         /* CHACHA20 limit (2^32 - 1) x 64 */
                                         template_job.msg_len_to_hash_in_bytes =
                                                 ((1ULL << 38) - 64) + 1;
@@ -665,7 +813,10 @@ test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
                              hash < IMB_AUTH_NUM; hash++) {
 
                                 switch (hash) {
-                                /* skip algos with that accept 0 len */
+                                /*
+                                 * Cases below don't allow for zero length
+                                 * hash messages
+                                 */
                                 case IMB_AUTH_HMAC_SHA_1:
                                 case IMB_AUTH_HMAC_SHA_224:
                                 case IMB_AUTH_HMAC_SHA_256:
@@ -674,13 +825,26 @@ test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
                                 case IMB_AUTH_MD5:
                                 case IMB_AUTH_KASUMI_UIA1:
                                         fill_in_job(&template_job, cipher, dir,
-                                                    hash, order);
+                                                    hash, order, &chacha_ctx,
+                                                    &gcm_ctx);
                                         template_job.msg_len_to_hash_in_bytes
                                                 = 0;
                                         break;
                                 default:
+                                        /*
+                                         * Skip algos that accept 0 length
+                                         * hash messages
+                                         */
                                         continue;
                                 }
+
+                                /*
+                                 * Skip hash algorithms belonging to AEAD
+                                 * algorithms, as the test is for authentication
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
+                                        continue;
+
                                 if (!is_submit_invalid(mb_mgr, &template_job,
                                                        TEST_AUTH_MSG_LEN_ZERO,
                                                        IMB_ERR_JOB_AUTH_LEN))
@@ -689,7 +853,7 @@ test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
                         }
 
         /* clean up */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         printf("\n");
@@ -702,17 +866,18 @@ test_job_invalid_mac_args(struct IMB_MGR *mb_mgr)
 static int
 test_job_invalid_cipher_args(struct IMB_MGR *mb_mgr)
 {
-        const JOB_HASH_ALG hash = IMB_AUTH_NULL;
-        JOB_CIPHER_DIRECTION dir;
-        JOB_CIPHER_MODE cipher;
-        JOB_CHAIN_ORDER order;
+        const IMB_HASH_ALG hash = IMB_AUTH_NULL;
+        IMB_CIPHER_DIRECTION dir;
+        IMB_CIPHER_MODE cipher;
+        IMB_CHAIN_ORDER order;
         struct IMB_JOB template_job;
-        struct IMB_JOB *job;
+        struct chacha20_poly1305_context_data chacha_ctx;
+        struct gcm_context_data gcm_ctx;
 
 	printf("Invalid JOB CIPHER arguments test:\n");
 
         /* prep */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         /*
@@ -727,8 +892,15 @@ test_job_invalid_cipher_args(struct IMB_MGR *mb_mgr)
                                     cipher == IMB_CIPHER_CUSTOM)
                                         continue;
 
+                                /*
+                                 * Skip cipher algorithms belonging to AEAD
+                                 * algorithms, as the test is for cipher
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
+                                        continue;
+
                                 fill_in_job(&template_job, cipher, dir,
-                                            hash, order);
+                                            hash, order, &chacha_ctx, &gcm_ctx);
                                 template_job.src = NULL;
                                 if (!is_submit_invalid(mb_mgr, &template_job,
                                                        TEST_CIPH_SRC_NULL,
@@ -749,8 +921,15 @@ test_job_invalid_cipher_args(struct IMB_MGR *mb_mgr)
                                     cipher == IMB_CIPHER_CUSTOM)
                                         continue;
 
+                                /*
+                                 * Skip cipher algorithms belonging to AEAD
+                                 * algorithms, as the test is for cipher
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
+                                        continue;
+
                                 fill_in_job(&template_job, cipher, dir,
-                                            hash, order);
+                                            hash, order, &chacha_ctx, &gcm_ctx);
                                 template_job.dst = NULL;
                                 if (!is_submit_invalid(mb_mgr, &template_job,
                                                        TEST_CIPH_DST_NULL,
@@ -771,8 +950,15 @@ test_job_invalid_cipher_args(struct IMB_MGR *mb_mgr)
                                     cipher == IMB_CIPHER_CUSTOM)
                                         continue;
 
+                                /*
+                                 * Skip cipher algorithms belonging to AEAD
+                                 * algorithms, as the test is for cipher
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
+                                        continue;
+
                                 fill_in_job(&template_job, cipher, dir,
-                                            hash, order);
+                                            hash, order, &chacha_ctx, &gcm_ctx);
                                 template_job.iv = NULL;
                                 if (!is_submit_invalid(mb_mgr, &template_job,
                                                        TEST_CIPH_IV_NULL,
@@ -790,7 +976,15 @@ test_job_invalid_cipher_args(struct IMB_MGR *mb_mgr)
                 for (cipher = IMB_CIPHER_CBC; cipher < IMB_CIPHER_NUM;
                      cipher++) {
                         fill_in_job(&template_job, cipher, IMB_DIR_ENCRYPT,
-                                    hash, order);
+                                    hash, order, &chacha_ctx, &gcm_ctx);
+
+                        /*
+                         * Skip cipher algorithms belonging to AEAD
+                         * algorithms, as the test is for cipher
+                         * only algorithms */
+                        if (check_aead(hash, cipher))
+                                continue;
+
                         switch (cipher) {
                         case IMB_CIPHER_NULL:
                         case IMB_CIPHER_CUSTOM:
@@ -814,8 +1008,15 @@ test_job_invalid_cipher_args(struct IMB_MGR *mb_mgr)
              order++)
                 for (cipher = IMB_CIPHER_CBC; cipher < IMB_CIPHER_NUM;
                      cipher++) {
+                        /*
+                         * Skip cipher algorithms belonging to AEAD
+                         * algorithms, as the test is for cipher
+                         * only algorithms */
+                        if (check_aead(hash, cipher))
+                                continue;
+
                         fill_in_job(&template_job, cipher, IMB_DIR_DECRYPT,
-                                    hash, order);
+                                    hash, order, &chacha_ctx, &gcm_ctx);
                         switch (cipher) {
                         case IMB_CIPHER_GCM:
                         case IMB_CIPHER_CBC:
@@ -878,17 +1079,30 @@ test_job_invalid_cipher_args(struct IMB_MGR *mb_mgr)
                                     cipher == IMB_CIPHER_CUSTOM)
                                         continue;
 
+                                /*
+                                 * Skip cipher algorithms belonging to AEAD
+                                 * algorithms, as the test is for cipher
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
+                                        continue;
+
                                 IMB_JOB *job = &template_job;
 
-                                fill_in_job(job, cipher, dir, hash, order);
+                                fill_in_job(job, cipher, dir, hash, order,
+                                            &chacha_ctx, &gcm_ctx);
 
                                 switch (cipher) {
                                 /* skip ciphers that allow msg length 0 */
                                 case IMB_CIPHER_GCM:
+                                case IMB_CIPHER_GCM_SGL:
                                 case IMB_CIPHER_CCM:
                                 case IMB_CIPHER_DOCSIS_SEC_BPI:
                                 case IMB_CIPHER_CHACHA20_POLY1305:
+                                case IMB_CIPHER_CHACHA20_POLY1305_SGL:
                                 case IMB_CIPHER_PON_AES_CNTR:
+                                case IMB_CIPHER_SNOW_V:
+                                case IMB_CIPHER_SNOW_V_AEAD:
+
                                         break;
                                 default:
                                         job->msg_len_to_cipher_in_bytes = 0;
@@ -912,21 +1126,33 @@ test_job_invalid_cipher_args(struct IMB_MGR *mb_mgr)
                                     cipher == IMB_CIPHER_CUSTOM)
                                         continue;
 
+                                /*
+                                 * Skip cipher algorithms belonging to AEAD
+                                 * algorithms, as the test is for cipher
+                                 * only algorithms */
+                                if (check_aead(hash, cipher))
+                                        continue;
+
                                 IMB_JOB *job = &template_job;
 
-                                fill_in_job(job, cipher, dir, hash, order);
+                                fill_in_job(job, cipher, dir, hash, order,
+                                            &chacha_ctx, &gcm_ctx);
 
                                 switch (cipher) {
                                         /* skip ciphers with no max limit */
                                 case IMB_CIPHER_GCM:
+                                case IMB_CIPHER_GCM_SGL:
                                 case IMB_CIPHER_CUSTOM:
                                 case IMB_CIPHER_CNTR:
                                 case IMB_CIPHER_CNTR_BITLEN:
                                 case IMB_CIPHER_PON_AES_CNTR:
+                                case IMB_CIPHER_SNOW_V:
+                                case IMB_CIPHER_SNOW_V_AEAD:
                                 case IMB_CIPHER_NULL:
                                         continue;
                                         /* not allowed with null hash */
                                 case IMB_CIPHER_CHACHA20_POLY1305:
+                                case IMB_CIPHER_CHACHA20_POLY1305_SGL:
                                         continue;
                                 case IMB_CIPHER_ZUC_EEA3:
                                         /* max is 8188 bytes */
@@ -966,8 +1192,39 @@ test_job_invalid_cipher_args(struct IMB_MGR *mb_mgr)
                                 printf(".");
                         }
 
+        /*
+         * OTHER MISC TESTS
+         */
+
+        /* CBCS NULL NEXT IV TEST */
+        for (order = IMB_ORDER_CIPHER_HASH; order <= IMB_ORDER_HASH_CIPHER;
+             order++)
+                for (dir = IMB_DIR_ENCRYPT; dir <= IMB_DIR_DECRYPT; dir++) {
+                        cipher = IMB_CIPHER_CBCS_1_9;
+
+                        /*
+                         * Skip cipher algorithms belonging to AEAD
+                         * algorithms, as the test is for cipher
+                         * only algorithms */
+                        if (check_aead(hash, cipher))
+                                continue;
+
+                        IMB_JOB *job = &template_job;
+
+                        fill_in_job(job, cipher, dir, hash, order, &chacha_ctx,
+                                    &gcm_ctx);
+
+                        job->cipher_fields.CBCS.next_iv = NULL;
+
+                        if (!is_submit_invalid(mb_mgr, job,
+                                               TEST_CIPH_NEXT_IV_NULL,
+                                               IMB_ERR_JOB_NULL_NEXT_IV))
+                                return 1;
+                        printf(".");
+                }
+
         /* clean up */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         printf("\n");
