@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2020, Intel Corporation
+ Copyright (c) 2020-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -221,7 +221,7 @@ chacha_job_ok(const struct IMB_JOB *job,
 {
         const int num = (const int)((uint64_t)job->user_data2);
 
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 printf("%d error status:%d, job %d",
                        __LINE__, job->status, num);
                 return 0;
@@ -254,7 +254,7 @@ test_chacha_many(struct IMB_MGR *mb_mgr,
               const unsigned text_len,
               const int dir,
               const int order,
-              const JOB_CIPHER_MODE cipher,
+              const IMB_CIPHER_MODE cipher,
               const int in_place,
               const int key_len,
               const int num_jobs)
@@ -282,7 +282,7 @@ test_chacha_many(struct IMB_MGR *mb_mgr,
         }
 
         /* flush the scheduler */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
@@ -333,7 +333,7 @@ test_chacha_many(struct IMB_MGR *mb_mgr,
         ret = 0;
 
  end:
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
 end_alloc:
@@ -351,7 +351,7 @@ test_chacha_vectors(struct IMB_MGR *mb_mgr,
                     struct test_suite_context *ctx,
                     const int vec_cnt,
                     const struct chacha_vector *vec_tab, const char *banner,
-                    const JOB_CIPHER_MODE cipher, const int num_jobs)
+                    const IMB_CIPHER_MODE cipher, const int num_jobs)
 {
 	int vect;
         DECLARE_ALIGNED(uint32_t enc_keys[15*4], 16);
