@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2019-2020, Intel Corporation
+ Copyright (c) 2019-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -36,9 +36,6 @@
 #include "gcm_ctr_vectors_test.h"
 #include "utils.h"
 
-#define SHA1_BLOCK_SIZE     64
-#define SHA1_DIGEST_SIZE    20
-
 int chained_test(struct IMB_MGR *mb_mgr);
 
 struct chained_vector {
@@ -56,8 +53,8 @@ struct chained_vector {
 };
 
 const struct test_set {
-        JOB_CIPHER_DIRECTION dir;
-        JOB_CHAIN_ORDER order;
+        IMB_CIPHER_DIRECTION dir;
+        IMB_CHAIN_ORDER order;
         const char *set_name;
 } test_sets[] = {
         {
@@ -145,7 +142,7 @@ chained_job_ok(const IMB_JOB *job,
                const uint8_t *padding,
                const size_t sizeof_padding)
 {
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 printf("%d error status:%d, job %d",
                        __LINE__, job->status, num_vec);
                 return 0;
@@ -213,10 +210,10 @@ test_chained_many(struct IMB_MGR *mb_mgr,
                   const void *enc_keys,
                   const void *dec_keys,
                   const struct chained_vector *vec,
-                  JOB_CIPHER_DIRECTION dir,
-                  JOB_CHAIN_ORDER order,
-                  JOB_CIPHER_MODE cipher,
-                  JOB_HASH_ALG hash,
+                  IMB_CIPHER_DIRECTION dir,
+                  IMB_CHAIN_ORDER order,
+                  IMB_CIPHER_MODE cipher,
+                  IMB_HASH_ALG hash,
                   const void *ipad_hash,
                   const void *opad_hash,
                   const unsigned in_place,
@@ -281,7 +278,7 @@ test_chained_many(struct IMB_MGR *mb_mgr,
         }
 
         /* flush the scheduler */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
@@ -363,7 +360,7 @@ test_chained_many(struct IMB_MGR *mb_mgr,
         ret = 0;
 
  end:
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
@@ -382,8 +379,8 @@ test_chained_vectors(struct IMB_MGR *mb_mgr,
                      struct test_suite_context *ctx,
                      const int vec_cnt,
                      const struct chained_vector *vec_tab, const char *banner,
-                     const JOB_CIPHER_MODE cipher,
-                     const JOB_HASH_ALG hash,
+                     const IMB_CIPHER_MODE cipher,
+                     const IMB_HASH_ALG hash,
                      unsigned hash_block_size, int num_jobs)
 {
         int vect;
@@ -504,7 +501,7 @@ chained_test(struct IMB_MGR *mb_mgr)
                                      chained_vectors,
                                      "AES-CBC + SHA1-HMAC standard test vectors",
                                      IMB_CIPHER_CBC, IMB_AUTH_HMAC_SHA_1,
-                                     SHA1_BLOCK_SIZE, num_jobs_tab[i]);
+                                     IMB_SHA1_BLOCK_SIZE, num_jobs_tab[i]);
 
         errors += test_suite_end(&ctx);
 
