@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2018-2020, Intel Corporation
+ Copyright (c) 2018-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -1084,7 +1084,7 @@ cmac_job_ok(const struct cmac_rfc4493_vector *vec,
 {
         const size_t auth_len = job->auth_tag_output_len_in_bytes;
 
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 printf("%d Error status:%d", __LINE__, job->status);
                 return 0;
         }
@@ -1170,7 +1170,7 @@ test_cmac(struct IMB_MGR *mb_mgr,
 		goto end;
         }
 
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         /**
@@ -1306,7 +1306,7 @@ test_cmac(struct IMB_MGR *mb_mgr,
         ret = 0;
 
  end:
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
@@ -1496,7 +1496,7 @@ cmac_test(struct IMB_MGR *mb_mgr)
         errors += test_suite_end(&ctx);
 
         /* CMAC 128 BITLEN with standard vectors */
-        test_suite_start(&ctx, "AES-CMAC-128-Bit-length");
+        test_suite_start(&ctx, "AES-CMAC-128-BIT-LENGTH");
         for (i = 1; i < 20; i++)
                 test_cmac_bitlen_std_vectors(mb_mgr, &ctx, i);
 
