@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2018-2020, Intel Corporation
+ Copyright (c) 2018-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -311,7 +311,7 @@ hmac_sha1_job_ok(const struct hmac_sha1_rfc2202_vector *vec,
                  const uint8_t *padding,
                  const size_t sizeof_padding)
 {
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 printf("line:%d job error status:%d ", __LINE__, job->status);
                 return 0;
         }
@@ -402,7 +402,7 @@ test_hmac_sha1(struct IMB_MGR *mb_mgr,
         IMB_SHA1_ONE_BLOCK(mb_mgr, buf, opad_hash);
 
         /* empty the manager */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
@@ -462,7 +462,7 @@ test_hmac_sha1(struct IMB_MGR *mb_mgr,
 
  end:
         /* empty the manager before next tests */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
