@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2009-2020, Intel Corporation
+ Copyright (c) 2009-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -87,60 +87,6 @@ struct kasumi_test_case kasumi_f9_func_tab[] = {
         {validate_kasumi_f9_user, "validate_kasumi_f9_user"}
 };
 
-static int membitcmp(const uint8_t *input, const uint8_t *output,
-                     const uint32_t bitoffset, const uint32_t bitlength)
-{
-        uint32_t bitresoffset;
-        uint8_t bitresMask = (uint8_t)-1 >> (bitoffset % CHAR_BIT);
-        uint32_t res;
-        uint32_t bytelengthfl = bitlength / CHAR_BIT;
-        const uint8_t *srcfl = input + bitoffset / CHAR_BIT;
-        const uint8_t *dstfl = output + bitoffset / CHAR_BIT;
-        int index = 1;
-
-        if (bitoffset % CHAR_BIT) {
-                if ((*srcfl ^ *dstfl) & bitresMask)
-                        return 1;
-                else {
-                        bytelengthfl--;
-                        srcfl++;
-                        dstfl++;
-                }
-        }
-        bitresoffset = (bitlength + bitoffset) % CHAR_BIT;
-        while (bytelengthfl--) {
-                res = *srcfl ^ *dstfl;
-                if (res) {
-                        if (bytelengthfl != 1)
-                                return index;
-                        else if (bitresoffset < CHAR_BIT) {
-                                if (res & ~((uint8_t)-1 << bitresoffset))
-                                        return index;
-                                else
-                                        res = 0;
-                        } else {
-                                srcfl++;
-                                dstfl++;
-                                index++;
-                        }
-                } else {
-                        srcfl++;
-                        dstfl++;
-                        index++;
-                }
-        }
-        if (bitresoffset > CHAR_BIT)
-                res = (*srcfl ^ *dstfl) &
-                      ~((uint8_t)-1 >> (bitresoffset % CHAR_BIT));
-        else if (bitresoffset == CHAR_BIT)
-                res = (*srcfl ^ *dstfl) &
-                      ~((uint8_t)-1 >> (bitoffset % CHAR_BIT));
-        else
-                res = 0;
-
-        return res;
-}
-
 static int
 submit_kasumi_f8_jobs(struct IMB_MGR *mb_mgr, kasumi_key_sched_t **keys,
                       uint64_t **ivs, uint8_t **src, uint8_t **dst,
@@ -170,7 +116,7 @@ submit_kasumi_f8_jobs(struct IMB_MGR *mb_mgr, kasumi_key_sched_t **keys,
                 job = IMB_SUBMIT_JOB(mb_mgr);
                 if (job != NULL) {
                         jobs_rx++;
-                        if (job->status != STS_COMPLETED) {
+                        if (job->status != IMB_STATUS_COMPLETED) {
                                 printf("%d error status:%d, job %d",
                                        __LINE__, job->status, i);
                                 return -1;
@@ -209,7 +155,7 @@ submit_kasumi_f9_job(struct IMB_MGR *mb_mgr, kasumi_key_sched_t *key,
 
         job = IMB_SUBMIT_JOB(mb_mgr);
         if (job != NULL) {
-                if (job->status != STS_COMPLETED) {
+                if (job->status != IMB_STATUS_COMPLETED) {
                         printf("%d error status:%d",
                                __LINE__, job->status);
                         return -1;
@@ -483,8 +429,8 @@ static int validate_kasumi_f8_1_bitblock(IMB_MGR *mgr, const unsigned job_api)
                                                    ciphBufAftPad, wrkBufAftPad,
                                                    bit_len, bit_offset);
 
-                if (membitcmp(wrkBufAftPad, plainBufAftPad, 0,
-                              kasumi_bit_vectors[i].LenInBits) != 0) {
+                if (membitcmp(wrkBufAftPad, plainBufAftPad,
+                              kasumi_bit_vectors[i].LenInBits, 0) != 0) {
                         printf("kasumi_f8_1_block(Dec) offset=0 vector:%d\n",
                                i);
                         hexdump(stdout, "Actual:", wrkBufAftPad, byte_len);
@@ -514,7 +460,7 @@ static int validate_kasumi_f8_1_bitblock(IMB_MGR *mgr, const unsigned job_api)
 
                 /* Check the ciphertext in the vector against the
                  * encrypted plaintext */
-                if (membitcmp(wrkBufAftPad, ciphBufAftPad, 4, bit_len) != 0) {
+                if (membitcmp(wrkBufAftPad, ciphBufAftPad, bit_len, 4) != 0) {
                         printf("kasumi_f8_1_block(Enc) offset=4  vector:%d\n",
                                i);
                         hexdump(stdout, "Actual:", wrkBufAftPad, byte_len);
@@ -532,8 +478,8 @@ static int validate_kasumi_f8_1_bitblock(IMB_MGR *mgr, const unsigned job_api)
                                                    ciphBufAftPad, wrkBufAftPad,
                                                    bit_len, bit_offset);
 
-                if (membitcmp(plainBufAftPad, plainBufAftPad, 4,
-                              bit_len) != 0) {
+                if (membitcmp(plainBufAftPad, plainBufAftPad,
+                              bit_len, 4) != 0) {
                         printf("kasumi_f8_1_block(Dec) offset=4 vector:%d\n",
                                i);
                         hexdump(stdout, "Actual:", wrkBufAftPad, byte_len);
@@ -576,6 +522,10 @@ static int validate_kasumi_f8_1_bitblock_offset(IMB_MGR *mgr,
         uint8_t *pWrkBuff = wrkbuf;
         uint64_t *pIV = &IV;
 
+        memset(srcBuff, 0, sizeof(srcBuff));
+        memset(dstBuff, 0, sizeof(dstBuff));
+        memset(wrkbuf, 0, sizeof(wrkbuf));
+
         if (!numKasumiTestVectors) {
                 printf("No Kasumi vectors found !\n");
                 return 1;
@@ -627,8 +577,8 @@ static int validate_kasumi_f8_1_bitblock_offset(IMB_MGR *mgr,
 
                 /* Check against the ciphertext in the vector against the
                  * encrypted plaintext */
-                ret = membitcmp(wrkbuf, dstBuff, offset,
-                                kasumi_bit_vectors->LenInBits[i]);
+                ret = membitcmp(wrkbuf, dstBuff,
+                                kasumi_bit_vectors->LenInBits[i], offset);
                 if (ret != 0) {
                         printf("kasumi_f8_1_block_linear(Enc)  vector:%d, "
                                "index:%d\n",
@@ -672,8 +622,8 @@ static int validate_kasumi_f8_1_bitblock_offset(IMB_MGR *mgr,
                         IMB_KASUMI_F8_1_BUFFER_BIT(mgr, pKeySched, IV, dstBuff,
                                                    wrkbuf, bit_len, offset);
 
-                ret = membitcmp(wrkbuf, srcBuff, offset,
-                                kasumi_bit_vectors->LenInBits[i]);
+                ret = membitcmp(wrkbuf, srcBuff,
+                                kasumi_bit_vectors->LenInBits[i], offset);
                 if (ret != 0) {
                         printf("kasumi_f8_1_block_linear(Dec)  "
                                "vector:%d,index:%d\n",
@@ -1225,7 +1175,7 @@ static int validate_kasumi_f8_n_blocks(IMB_MGR *mgr, const unsigned job_api)
         for (i = 0; i < NUM_SUPPORTED_BUFFERS; i++) {
                 bitOffsets[i] = 0;
 
-                key[i] = malloc(KASUMI_KEY_SIZE);
+                key[i] = malloc(IMB_KASUMI_KEY_SIZE);
                 if (!key[i]) {
                         printf("malloc(key[%u]:failed !\n", i);
                         goto exit;
@@ -1248,7 +1198,7 @@ static int validate_kasumi_f8_n_blocks(IMB_MGR *mgr, const unsigned job_api)
                         goto exit;
                 }
 
-                memset(key[i], 0xAA, KASUMI_KEY_SIZE);
+                memset(key[i], 0xAA, IMB_KASUMI_KEY_SIZE);
                 if (IMB_KASUMI_INIT_F8_KEY_SCHED(mgr, key[i], pKeySched[i])) {
                         printf("IMB_KASUMI_INIT_F8_KEY_SCHED() error\n");
                         goto exit;
@@ -1484,7 +1434,7 @@ static int validate_kasumi_f9(IMB_MGR *mgr, const unsigned job_api)
         uint8_t *pKey = NULL;
         int keyLen = 16;
         uint8_t srcBuff[MAX_DATA_LEN];
-        uint8_t digest[KASUMI_DIGEST_SIZE];
+        uint8_t digest[IMB_KASUMI_DIGEST_SIZE];
         int numKasumiF9TestVectors, i;
         hash_test_vector_t *kasumiF9_test_vectors = NULL;
         int ret = 1;
@@ -1521,7 +1471,7 @@ static int validate_kasumi_f9(IMB_MGR *mgr, const unsigned job_api)
                 memcpy(srcBuff, kasumiF9_test_vectors[i].input, byteLen);
 
                 memcpy(digest, kasumiF9_test_vectors[i].exp_out,
-                       KASUMI_DIGEST_SIZE);
+                       IMB_KASUMI_DIGEST_SIZE);
 
                 if (IMB_KASUMI_INIT_F9_KEY_SCHED(mgr, pKey, pKeySched)) {
                         printf("IMB_KASUMI_INIT_F9_KEY_SCHED()error\n");
@@ -1538,11 +1488,12 @@ static int validate_kasumi_f9(IMB_MGR *mgr, const unsigned job_api)
 
                 /* Compare the digest with the expected in the vectors */
                 if (memcmp(digest, kasumiF9_test_vectors[i].exp_out,
-                           KASUMI_DIGEST_SIZE) != 0) {
-                        hexdump(stdout, "Actual", digest, KASUMI_DIGEST_SIZE);
+                           IMB_KASUMI_DIGEST_SIZE) != 0) {
+                        hexdump(stdout, "Actual", digest,
+                                IMB_KASUMI_DIGEST_SIZE);
                         hexdump(stdout, "Expected",
                                 kasumiF9_test_vectors[i].exp_out,
-                                KASUMI_DIGEST_SIZE);
+                                IMB_KASUMI_DIGEST_SIZE);
                         printf("F9 integrity %d Failed\n", i);
                         goto exit;
                 }
@@ -1571,7 +1522,7 @@ static int validate_kasumi_f9_user(IMB_MGR *mgr, const unsigned job_api)
 
         uint64_t iv[MAX_IV_LEN];
         uint8_t srcBuff[MAX_DATA_LEN];
-        uint8_t digest[KASUMI_DIGEST_SIZE];
+        uint8_t digest[IMB_KASUMI_DIGEST_SIZE];
         uint32_t direction;
         int ret = 1;
 
@@ -1619,10 +1570,11 @@ static int validate_kasumi_f9_user(IMB_MGR *mgr, const unsigned job_api)
 
                 /* Compare the digest with the expected in the vectors */
                 if (memcmp(digest, kasumiF9_vectors[i].exp_out,
-                           KASUMI_DIGEST_SIZE) != 0) {
-                        hexdump(stdout, "digest", digest, KASUMI_DIGEST_SIZE);
+                           IMB_KASUMI_DIGEST_SIZE) != 0) {
+                        hexdump(stdout, "digest", digest,
+                                IMB_KASUMI_DIGEST_SIZE);
                         hexdump(stdout, "exp_out", kasumiF9_vectors[i].exp_out,
-                                KASUMI_DIGEST_SIZE);
+                                IMB_KASUMI_DIGEST_SIZE);
                         printf("direction %d\n", direction);
                         printf("F9 integrity %d Failed\n", i);
                         goto exit;
