@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2020, Intel Corporation
+ Copyright (c) 2020-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -416,7 +416,7 @@ poly1305_job_ok(struct IMB_MGR *mb_mgr,
 {
         const size_t auth_len = job->auth_tag_output_len_in_bytes;
 
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 const int errcode = imb_get_errno(mb_mgr);
 
                 printf("Error!: job status %d, errno %d => %s\n",
@@ -480,7 +480,7 @@ test_poly1305(struct IMB_MGR *mb_mgr,
         }
 
 
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         /**
@@ -528,7 +528,7 @@ test_poly1305(struct IMB_MGR *mb_mgr,
         ret = 0;
 
  end:
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
