@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2019-2020, Intel Corporation
+ Copyright (c) 2019-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -470,7 +470,7 @@ test_pon(struct IMB_MGR *mb_mgr,
                 memcpy(target + sizeof(padding), out_text, len_to_bip);
         }
 
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         job = IMB_GET_NEXT_JOB(mb_mgr);
@@ -485,7 +485,7 @@ test_pon(struct IMB_MGR *mb_mgr,
         if (iv != NULL) {
                 job->enc_keys = expkey;
                 job->dec_keys = expkey;
-                job->key_len_in_bytes = IMB_KEY_AES_128_BYTES;
+                job->key_len_in_bytes = IMB_KEY_128_BYTES;
                 job->iv = iv;
                 job->iv_len_in_bytes = 16;
                 job->msg_len_to_cipher_in_bytes = (uint64_t) len_to_cipher;
@@ -510,7 +510,7 @@ test_pon(struct IMB_MGR *mb_mgr,
                 goto end;
         }
 
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 printf("%d Error status:%d", __LINE__, job->status);
                 goto end;
         }
@@ -612,7 +612,7 @@ test_pon(struct IMB_MGR *mb_mgr,
         /* all checks passed */
         ret = 0;
 
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
  end:
         if (target != NULL)
