@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2018-2020, Intel Corporation
+ Copyright (c) 2018-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -399,7 +399,7 @@ sha_job_ok(const struct sha_vector *vec,
            const uint8_t *padding,
            const size_t sizeof_padding)
 {
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 printf("line:%d job error status:%d ", __LINE__, job->status);
                 return 0;
         }
@@ -463,7 +463,7 @@ test_sha(struct IMB_MGR *mb_mgr,
         }
 
         /* empty the manager */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
@@ -522,7 +522,7 @@ test_sha(struct IMB_MGR *mb_mgr,
 
  end:
         /* empty the manager before next tests */
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
