@@ -1,5 +1,5 @@
 /*****************************************************************************
- Copyright (c) 2020, Intel Corporation
+ Copyright (c) 2020-2021, Intel Corporation
 
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
@@ -312,7 +312,7 @@ xcbc_job_ok(const struct xcbc_rfc3566_vector *vec,
 {
         const size_t auth_len = job->auth_tag_output_len_in_bytes;
 
-        if (job->status != STS_COMPLETED) {
+        if (job->status != IMB_STATUS_COMPLETED) {
                 printf("%d Error status:%d", __LINE__, job->status);
                 return 0;
         }
@@ -376,7 +376,7 @@ test_xcbc(struct IMB_MGR *mb_mgr,
 
         IMB_AES_XCBC_KEYEXP(mb_mgr, vec->key, k1_exp, k2, k3);
 
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         /**
@@ -470,7 +470,7 @@ test_xcbc(struct IMB_MGR *mb_mgr,
         ret = 0;
 
  end:
-        while ((job = IMB_FLUSH_JOB(mb_mgr)) != NULL)
+        while (IMB_FLUSH_JOB(mb_mgr) != NULL)
                 ;
 
         for (i = 0; i < num_jobs; i++) {
